package brownstrategy;

import java.util.Stack;

import brownstrategy.agent.Agent;
import strategy.UtilityBasedStrategy;
import util.Array;
import game.Game;

/**
 * A strategy based on (Zinkevich, 2003) (gradient descent).
 * @author maz
 *
 * edited by jberg
 *
 */
public class BrownGIGA extends BrownUtilityBasedStrategy{
	double[] currentStrategy;
	double learningRate;
	Stack<double[]> currStratStack;
	public BrownGIGA(Game g, long seed, String options) {
		super(g,seed,options);
		currentStrategy = new double[numActions];
		projectCurrentStrategy();
		learningRate = 0.001;
		currStratStack = new Stack<double[]>();
	}

	@Override
	public void observeUtilities(int[] possibleUtilities, int currentAction) {
		for(int i=0;i<currentStrategy.length;i++){
			currentStrategy[i]+=possibleUtilities[i]*learningRate;
		}
		projectCurrentStrategy();
	}

	public void projectCurrentStrategy(){
		projectVector(currentStrategy);
	}

	public void setVector(double[] vector){
		System.arraycopy(vector, 0, currentStrategy, 0, currentStrategy.length);
	}

	public double[] getVector(){
		return currentStrategy;
	}

	public static void projectVector(double[] currentStrategy) {
		double total=0;
		// The actions that are still being used.
		boolean[] stillLive = new boolean[currentStrategy.length];
		for(int i=0;i<currentStrategy.length;i++){
			stillLive[i]=true;
		}
		int numStillLive = currentStrategy.length;

		for(int i=0;i<currentStrategy.length;i++){
			total+=currentStrategy[i];
		}

		double necessaryChange = (total-1.0)/((double)numStillLive);

		while(true){
			double amountBelow = 0;
			for(int i=0;i<currentStrategy.length;i++){
				if (stillLive[i]){
					currentStrategy[i]-=necessaryChange;
					if (currentStrategy[i]<0){
						amountBelow-=currentStrategy[i];
						currentStrategy[i]=0;
						stillLive[i]=false;
						numStillLive--;
					}
				} 
			}
			if (amountBelow==0){
				break;
			}
			necessaryChange = amountBelow/((double)numStillLive);
		}
	}

	@Override
	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		_cacheDist = currentStrategy.clone();
		return _cacheDist;
	}

	@Override
	public Agent getCopy() {
		return new BrownGIGA(_g, _seed, _options);
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new BrownGIGA(g, seed, options);
	}

	@Override
	public void lookAhead(int[] actions) {
		currStratStack.push(currentStrategy);
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		currentStrategy = currStratStack.pop();
		_day--;
	}
	
}
