package brownstrategy;

import java.util.Stack;

import brownstrategy.agent.Agent;
import strategy.UtilityBasedStrategy;
import game.Game;



/**
 * The internal regret matching strategy.
 * @author maz
 * 
 * edited by jberg
 *
 */
public class BrownInternalRegretMatchingStrategy extends BrownUtilityBasedStrategy{
	/**
	 * regrets[i][j] is the regret of not playing j instead of i.
	 */ 
	int[][] regrets;
	int[] offsets;
	Stack<int[][]> regretsStack;
	Stack<int[]> offsetStack;
	public BrownInternalRegretMatchingStrategy(Game g, long seed, String options) {
		super(g,seed,options);
		this.regrets = new int[numActions][numActions];
		this.offsets = new int[numActions];
		regretsStack = new Stack<int[][]>();
		offsetStack = new Stack<int[]>();
	}



	public int[][] getRegrets(){
		return regrets;
	}

	public void setRegrets(int[][] values){
		assert(regrets.length==numActions);
		for(int i=0;i<numActions;i++){
			assert(regrets[i].length==numActions);
			for(int j=0;j<numActions;j++){
				regrets[i][j]=values[i][j];
			}

		}

	}

	/**
	 * Gets the maximum total positive regret of an action.
	 * The total positive regret for an action is the sum of the
	 * regrets of all the actions that one could have played instead
	 * that would have done better.
	 * @return the maximum total positive regret of an action
	 */
	private int getMaxRegret() {
		int maxSoFar = 0;
		assert(regrets.length==numActions);
		for(int i=0;i<numActions;i++){
			int total=0;
			assert(regrets[i].length==numActions);
			for(int j=0;j<numActions;j++){
				if (regrets[i][j]>0){
					total+=regrets[i][j];
				}
			}
			if (total>maxSoFar){
				maxSoFar = total;
			}

		}
		return maxSoFar;
	}



	@Override
	public void observeUtilities(int[] possibleUtilities, int actionTaken) {
		for(int i=0;i<possibleUtilities.length;i++){
			regrets[actionTaken][i] += possibleUtilities[i]-possibleUtilities[actionTaken];
			offsets[actionTaken] += possibleUtilities[actionTaken];
		}
	}



	@Override
	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		double[] actions = new double[offsets.length];
		for(int i = 0; i < numActions; i++) {
			for(int j = 0; j < numActions; j++) {
				actions[j] += regrets[i][j] + offsets[i];
			}
		}
		_cacheDist = normalize(actions);
		return _cacheDist;
	}



	@Override
	public Agent getCopy() {
		return new BrownInternalRegretMatchingStrategy(_g, _seed, _options);
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new BrownInternalRegretMatchingStrategy(g, seed, options);
	}

	@Override
	public void lookAhead(int[] actions) {
		regretsStack.push(regrets);
		offsetStack.push(offsets);
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		regrets = regretsStack.pop();
		offsets = offsetStack.pop();
		_day--;
	}

}
