package brownstrategy;

import game.Game;
import game.Strategy;

import java.util.Random;

import brownstrategy.agent.Agent;
import brownstrategy.agent.DeterministicAgent;
import brownstrategy.agent.NondeterministicAgent;

/**
 * 
 * This contains every length 2 subsequence of moves
 * possible.  It is called a de bruijn sequence.
 * This agent picks a random starting point in the
 * de bruijn sequence and just plays through.
 * 
 * @author jberg
 *
 */
public class DeBruijn2 extends NondeterministicAgent {
	/**
	 * The number of actions available in the game.
	 */
	int numActions;

	/**
	 * The last actions that were observed
	 */
	int[] lastActions;
	
	/**
	 * Number of rounds thus far
	 */
	int numRounds;
	
	/**
	 * Random starting point in the de bruijn sequence
	 */
	int deBruijnStart;

	/**
	 * A random number generator for the random actions.
	 */
	Random r;
	public DeBruijn2(Game g, long seed, String options) {
		super(g,seed,options);
		this.numActions = g.getNumActions();
		this.lastActions = null;
		this.r = new Random(seed);
		numRounds = 0;
		deBruijnStart = r.nextInt(deBruijn.length);
	}

	int[] deBruijn = {8, 10, 8, 7, 6, 5, 4, 3, 2, 1, 9, 10, 9, 1, 1, 0, 11, 0, 10, 0, 8, 8, 0, 7, 
			7, 0, 6, 6, 0, 5, 5, 0, 4, 4, 0, 3, 3, 0, 2, 9, 2, 2, 0, 9, 9, 11, 10, 10, 
			11, 11, 9, 0, 0, 1, 11, 1, 10, 1, 8, 1, 7, 1, 6, 1, 5, 1, 4, 1, 3, 9, 3, 1, 
			2, 11, 2, 10, 2, 8, 2, 7, 2, 6, 2, 5, 2, 4, 9, 4, 2, 3, 11, 3, 10, 3, 8, 3, 
			7, 3, 6, 3, 5, 9, 5, 3, 4, 11, 4, 10, 4, 8, 4, 7, 4, 6, 9, 6, 4, 5, 11, 5, 
			10, 5, 8, 5, 7, 9, 7, 5, 6, 11, 6, 10, 6, 8, 9, 8, 6, 7, 11, 7, 10, 7, 8, 11};
	
	@Override
	public Agent getCopy() {
		return new DeBruijn2(_g, _seed, _options);
	}

	@Override
	public Agent getCopy(Game g, long seed, String options) {
		return new DeBruijn2(g, seed, options);
	}
	
	
	@Override
	public double[] getActionDistribution() {
		if(_day == _cacheDay) {
			return _cacheDist;
		}
		_cacheDist = new double[12];
		_cacheDay = _day;
		int action = deBruijn[(deBruijnStart + numRounds) % deBruijn.length];
		_cacheDist[action] = 1;
		return _cacheDist;
	}

	@Override
	public void observeOutcome(int[] actions) {
		lastActions = actions;
		_day++;
		numRounds++;
	}

	@Override
	public void lookAhead(int[] actions) {
		observeOutcome(actions);
	}

	@Override
	public void lookBack() {
		_day--;
		numRounds--;
	}

}
