package brownstrategy.oracles;

import game.Game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

import util.Array;

/**
 *
 * This oracle makes its predictions based on each agents
 * opponents moves.  For any given number of last moves
 * it checks for matching sequences in the agents history
 * and determines how it moved after that sequence of moves
 * previously seen.  This oracle also takes into consider
 * the rotational symmetry of the game
 * 
 * @author jberg
 *
 */

public class PrevSequenceAllRotandOppSymOracle extends AbstractPrevSequenceOracle {

	public PrevSequenceAllRotandOppSymOracle(Game g, long seed, String options) {
		super(g,seed,options);
	}

	public int[] getOppDist(int opp) {
		int[] arr = new int[12];
		ArrayList<Integer> opp1,opp2;
		ArrayList<Integer> playerBeingModeled;
		if (opp == 0) {
			opp1 = player2Moves;
			opp2 = player3Moves;
			playerBeingModeled = player1Moves;
		}
		else if (opp == 1) {
			opp1 = player1Moves;
			opp2 = player3Moves;
			playerBeingModeled = player2Moves;
		}
		else {
			opp1 = player1Moves;
			opp2 = player2Moves;
			playerBeingModeled = player3Moves;
		}
		if(_numLastMoves == 0 || playerBeingModeled.size() <= _numLastMoves) {
			for (Integer move : playerBeingModeled) {
				arr[move] = arr[move] + 1;
			}
		}
		else {
			List<Integer> prevMoves1 = opp1.subList(opp1.size() - _numLastMoves, opp1.size());
			List<Integer> prevMoves2 = opp2.subList(opp2.size() - _numLastMoves, opp2.size());
			List<Integer> prevMoves3 = playerBeingModeled.subList(playerBeingModeled.size() - _numLastMoves, playerBeingModeled.size());
			for(int i = 0; i < opp1.size() - _numLastMoves; i++) {
				List<Integer> subMoves1 = opp1.subList(i, i+_numLastMoves);
				List<Integer> subMoves2 = opp2.subList(i, i+_numLastMoves);
				List<Integer> subMoves3 = playerBeingModeled.subList(i, i+_numLastMoves);
				/*
				 * This checks to see if the distance between the two opponents
				 * is the same
				 */
				if (arrListEaualsOppSym(getDiffList(prevMoves3, prevMoves1), getDiffList(subMoves3, subMoves1),
						getDiffList(prevMoves3, prevMoves2), getDiffList(subMoves3, subMoves2))) {
					int idx = (playerBeingModeled.get(playerBeingModeled.size()-1) + ((playerBeingModeled.get(i+_numLastMoves) - playerBeingModeled.get(i+_numLastMoves-1) + 12) % 12)) % 12;
					arr[idx] = arr[idx] + 1;
				}
			}
		}
		return arr;
	}
	
	public List<Integer> getDiffList(List<Integer> arrList1, List<Integer> arrList2) {
		ArrayList<Integer> diffList = new ArrayList<Integer>();
		for(int i = 0; i < arrList1.size(); i++) {
			diffList.add((arrList1.get(i) - arrList2.get(i) + 12) % 12);
		}
		return diffList;
	}
	
	public boolean arrListEaualsOppSym(List<Integer> prevMoves1, List<Integer> prevMoves2, 
			List<Integer> prevMoves3, List<Integer> prevMoves4) {
		for(int i = 0; i < prevMoves1.size(); i++) {
			if(prevMoves1.get(i) == prevMoves2.get(i)) {
				if(prevMoves3.get(i) != prevMoves4.get(i)) {
					return false;
				}
				//Otherwise it is a match and we continue
			}
			else if(prevMoves1.get(i) == prevMoves4.get(i)) {
				if(prevMoves3.get(i) != prevMoves2.get(i)) {
					return false;
				}
				//Otherwise it is a match and we continue
			}
			else {
				return false;
			}
		}
		return true;
	}

	
	public String toString() {
		return "PrevSequenceAllRotandOppSymOracle(n = " + _numLastMoves + ")";
	}

	public PrevSequenceAllRotandOppSymOracle getCopy() {
		return new PrevSequenceAllRotandOppSymOracle(_g,_seed,_options);
	}
	
	public PrevSequenceAllRotandOppSymOracle getCopy(Game g,long seed,String options) {
		return new PrevSequenceAllRotandOppSymOracle(_g,_seed,_options);
	}
	
}