package brownstrategy.oracles;

import game.Game;

import java.util.ArrayList;
import java.util.List;
import java.util.Random;

/**
 *
 * This oracle makes its predictions based on each agents
 * opponents moves.  For any given number of last moves
 * it checks for matching sequences in the agents history
 * and determines how it moved after that sequence of moves
 * previously seen.
 * 
 * @author jberg
 *
 */

public class PrevSequenceOpponentsOracle extends AbstractPrevSequenceOracle {

	public PrevSequenceOpponentsOracle(Game g, long seed, String options) {
		super(g,seed,options);
	}

	public int[] getOppDist(int opp) {
		int[] arr = new int[12];
		ArrayList<Integer> opp1,opp2;
		ArrayList<Integer> playerBeingModeled;
		if (opp == 0) {
			opp1 = player2Moves;
			opp2 = player3Moves;
			playerBeingModeled = player1Moves;
		}
		else if (opp == 1) {
			opp1 = player1Moves;
			opp2 = player3Moves;
			playerBeingModeled = player2Moves;
		}
		else {
			opp1 = player1Moves;
			opp2 = player2Moves;
			playerBeingModeled = player3Moves;
		}
		if(_numLastMoves == 0 || playerBeingModeled.size() <= _numLastMoves) {
			for (Integer move : playerBeingModeled) {
				arr[move] = arr[move] + 1;
			}
		}
		else {
			List<Integer> prevMoves = opp1.subList(opp1.size() - _numLastMoves, opp1.size());
			List<Integer> ourPrevMoves = opp2.subList(opp2.size() - _numLastMoves, opp2.size());
			for(int i = 0; i < opp1.size() - _numLastMoves; i++) {
				List<Integer> subMoves = opp1.subList(i, i+_numLastMoves);
				List<Integer> ourSubMoves = opp2.subList(i, i+_numLastMoves);
				if (arrListEquals(prevMoves, subMoves) && arrListEquals(ourPrevMoves, ourSubMoves)) {
					arr[playerBeingModeled.get(i+_numLastMoves)] = arr[playerBeingModeled.get(i+_numLastMoves)] + 1;
				}
			}
		}
		return arr;
	}
	
	public String toString() {
		return "PrevSequenceOpponentsOracle(n = " + _numLastMoves + ")";
	}

	public PrevSequenceOpponentsOracle getCopy() {
		return new PrevSequenceOpponentsOracle(_g,_seed,_options);
	}
	
	@Override
	public PrevSequenceOpponentsOracle getCopy(Game g, long seed, String options) {
		return new PrevSequenceOpponentsOracle(g,seed,options);
	}
	
}