package strategy;

import strategy.Model;
import java.util.ArrayList;
//import java.util.*;
import java.util.ListIterator;
//import static java.lang.Math.pow;

/* Author: Dustin Morrill
 * Date: August 25, 2010
 *
 * Agent description:
 * This is the player information storage object used in my adaptive agents
 * that play the lemonade stand game.
 *
 */

public class Player extends Model {
    ArrayList<Integer> pos;
    ArrayList<Integer> latest_util;
    ArrayList<Integer> total_util;
    Model model;
    int label;

    public Player (int lbl) {
        pos = new ArrayList<Integer>();
        latest_util = new ArrayList<Integer>();
        total_util = new ArrayList<Integer>();
        Integer init_total_util = new Integer(0);
        total_util.add(init_total_util);
        model = new Model();
        label = lbl;
    }

    public void setPos (int int_pos) {
        Integer Integer_pos = new Integer(int_pos);
        pos.add(Integer_pos);
    }

    public int getPos (int index) {
        Integer Integer_index = new Integer(index-1);
        return pos.get(Integer_index);
    }

    public void setLatestUtil (int util) {
        Integer Integer_util = new Integer(util);
        latest_util.add(Integer_util);
    }

    public int getLatestPos () {
        return pos.get(pos.size()-1);
    }

    public int getLatestUtil () {
        return latest_util.get(latest_util.size()-1).intValue();
    }

    public int[] getLatestUtilArray () {
        int[] a = new int[latest_util.size()];
        for (int i = 0; i < latest_util.size(); i++) {
            a[i] = latest_util.get(i).intValue();
        }
        return a;
    }

    public int[] getLatestUtilArray (int len) {
        int[] a = getLatestUtilArray();
        int[] a_2 = new int[len];
        System.arraycopy(a, a.length-len, a_2, 0, len);
        return a_2;
    }

    public void updateTotalUtil (int new_util) {
        int util = total_util.get(total_util.size()-1).intValue() + new_util;
        Integer Integer_util = new Integer(util);
        total_util.add(Integer_util);
    }

    public int getTotalUtil () {
        return total_util.get(total_util.size()-1).intValue();
    }

    public boolean checkConstant (int threshold) {
        // Check if the player is acting constantly    
        boolean is_constant = true;
        //System.out.printf("Entering loop\n");
        int limit = (pos.size() - threshold);
        //System.out.printf("limit: %d, size: %d, threshold: %d\n", limit, pos.size(),  threshold);
        for (int i = pos.size(); i > limit && i > 1 && is_constant; i--) {
            //System.out.printf("i: %d\n", i);
            if (getPos(i) != getPos(i-1)) {
                is_constant = false;
            }
        }
        //System.out.printf("Exiting loop\n");
        return is_constant;
    }

    public int getConstantScore () {
        int constant_score = 0;
        // This "constant score" could be calculated a lot of ways, but I'm
        // going to try computing it as the number of times an agent plays
        // at a particular position starting from the most recent actions
        for (int i = pos.size(); i > 1 && getPos(i) == getPos(i-1); i--) {
            constant_score++;
        }
        //System.out.printf("constant_score: %d\n", constant_score);
        return constant_score;
    }

    public void setConstant (boolean val) {
        model.setIsConstant(val);
        // If the player is constant it can't be random
        if (val && isRandom()) {
            model.setIsRandom(!val);
        }
    }

    public boolean isConstant () {
        return model.getIsConstant();
    }

    public void setRandom (boolean val) {
        model.setIsRandom(val);
        if (isConstant()) {
            setConstant(false);
        }
    }

    public boolean isRandom () {
        return model.getIsRandom();
    }

    public void setDefensive () {
        model.setIsDefensive(true);
    }

    public boolean isDefensive () {
        return model.getIsDefensive();
    }

    public void setOffensive () {
        model.setIsOffensive(true);
    }

    public boolean isOffensive () {
        return model.getIsOffensive();
    }

    public int getLabel () {
        return label;
    }
}
