/* Authors : David Reitter, Ion Juvina, Unmesh Kurup, et al.
   (Also includes Soton, RL3, ACTR from the 2009 competition)
 */

package edu.cmu.psy.reitter;

import game.Game;
import game.Strategy;
import game.Utility;
import java.util.Random;
import lemonade.LemonadeUtility;
import static java.lang.Math.*;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.*;

import game.Strategy.*;

import edu.cmu.psy.reitter.meta.LM;
import edu.cmu.psy.reitter.meta.*;

public class Meta
    extends Taylor
    implements Strategy {

    final boolean GATHER_STATS = false;  // when true, also change RunLemonadeTournament to have first model play against the others all the time.
    final boolean LOG = false;

    final boolean USE_LMS = true;
    final boolean USE_SDEV_CUTOFF = false;
    
    final int NUM_STRATEGIES = 20;

    Utility utility;
    Random mRandom;

    int meta_strategy = -1;  // -1 means choose best

    LM[] strat_lm;

    int[] hist_strategies;
    int[] hist_payoffs;
    int[] hist_steps;
    int current_step = -1;
    int current_record = -1;
    int[] latest_actions;

    int[] initial_strategy_preference;
    int[] deferred_strategy_preference;
    boolean[] observe_outcome_hypothetical;
    String[] strat_name;
    
    int total_payoff;

    Strategy[] strategies = new Strategy[NUM_STRATEGIES];
    int num_strategies;
    int latest_strategy;
    int preferred_strategy = 0;

    void registerStrategy(Strategy s, int initial_pref, int deferred_pref, boolean hypothetical_outcome)
    {
	strategies[++num_strategies] = s;
	initial_strategy_preference[num_strategies] = initial_pref;
	deferred_strategy_preference[num_strategies] = deferred_pref;
	observe_outcome_hypothetical[num_strategies] = hypothetical_outcome;
	strat_name[num_strategies] = s.getClass().getName();
    }


    public Meta (Game g, long seed, String options) throws Exception
    //throws java.lang.ClassNotFoundException java.lang.InstantiationException
    {
	super(g, seed, options);
      utility = g.getUtility();
      mRandom = new Random(seed);
      
      preferred_strategy = 0;
      initial_strategy_preference = new int[NUM_STRATEGIES];
      deferred_strategy_preference = new int[NUM_STRATEGIES];
      observe_outcome_hypothetical = new boolean[NUM_STRATEGIES];
      strat_name = new String[NUM_STRATEGIES];
      num_strategies = -1;
      //
      registerStrategy(new StickySharp(g, seed, options), 200, -50, true);   // internal (by Ion Juvina).
      if (GATHER_STATS || USE_LMS)
	  registerStrategy(new ConstantStrategy_PlusRandom(g, seed, options), 25, -50, true);  // internal (by Unmesh Kurup).
      registerStrategy(new EAsquared(g, seed, options), 20, 0, true);   // de Cote.  Soton from 2009 Comp
      registerStrategy(new CoOpp(g, seed, options), 0, 0, false);  // Wunder et al, RL3 from 2009 comp

      if (GATHER_STATS)
	  {
	      // first group: when learning
      // registerStrategy(new strategy.ACTR(g, seed, options), 0, 0, false);
      // registerStrategy(new brownstrategy.PerrinStrategy(g, seed, options), 0, 0, false);
      // registerStrategy(new strategy.Strategist(g, seed, options), 0, 0, false);
      // registerStrategy(new strategy.meta.FrozenPontiac(g, seed, options), 0, 0, false);
      // registerStrategy(new strategy.meta.ModifiedConstant(g, seed, options), 0, 0, false);  // Pujara from 2009 comp
      // registerStrategy(new strategy.ConstantStrategy(g, seed, options), 0, 0, false);
      // registerStrategy(new strategy.ExternalRegretMatchingStrategy(g, seed, options), 0, 0, false);
      // registerStrategy(new strategy.InternalRegretMatchingStrategy(g, seed, options), 0, 0, false);
      // registerStrategy(new strategy.GIGA(g, seed, options), 0, 0, false);
      // registerStrategy(new strategy.UniformStrategy(g, seed, options), 0, 0, false);


      //registerStrategy(new brownstrategy.MimicOpponent(g, seed, options), 0, 0, false);
      
      // registerStrategy(new brownstrategy.DeBruijn2(g, seed, options), 0, 0, false);
      //registerStrategy(new brownstrategy.DeBruijn3(g, seed, options), 0, 0, false);
      // registerStrategy(new brownstrategy.MimicOpponentPlus6(g, seed, options), 0, 0, false);

	  }

      if (GATHER_STATS)
	  meta_strategy = mRandom.nextInt(num_strategies+1);

      hist_strategies = new int[3000];
      hist_payoffs = new int[3000];
      hist_steps = new int[3000];
      latest_actions = new int[NUM_STRATEGIES];
      total_payoff = 0;
      

      if (!GATHER_STATS && USE_LMS)
	  initLMs();
  }

  @Override
  public int getAction() {

      
 
       double temperature = 0.2;
       double d = 1.3;//.7;

      current_step++;


      /* Calculate utilities of each action */

     int best_strategy = 0;
      if (meta_strategy>-1)
	  {
	      if (current_step < 15)
		  best_strategy = 1; // constant
	      else
		  best_strategy = meta_strategy;
	      latest_actions[best_strategy] = strategies[best_strategy].getAction();
	  }
      else if (USE_LMS)
	  {
	      
	      for(int i=num_strategies; i>=0; i--)
		  {
		      latest_actions[i] = -1;
		      if (meta_strategy==-1 || i==meta_strategy)
			  latest_actions[i] = strategies[i].getAction();
		      if (latest_actions[i] < 0)
			  latest_actions[i] = -1;  // the strategy doesn't know
		  }


	      if (current_step < 30 && false)
		  best_strategy = 0;
	      else
		  {
		      float estd[] = estimatePayoffs();
		      double best_payoff = -1;
		      double sum = 0.0;
		      for(int i=num_strategies; i>=0; i--)
			  {
			       // System.out.format("%s=%f ", strat_name[i], estd[i]);
			      if ( estd[i]>best_payoff)
				  {
				      best_payoff = estd[i];
				      best_strategy = i;
				  }
			      if (USE_SDEV_CUTOFF)
				  sum += estd[i];
			  }
		      double mean = (double)sum/(num_strategies+1);
		      // calc standard deviations
		      double sdev = 0;
		      if (USE_SDEV_CUTOFF)
			  {
			      for(int i=num_strategies; i>=0; i--)
				  sdev += (double) pow(estd[i]-mean, 2);
			      sdev = pow((double)sdev/(num_strategies), 0.5);
			  }

		      if (USE_SDEV_CUTOFF)
			  {
			      // if best strategy isn't at least 2 standard deviations away, choose sticky sharp
			      // System.out.format("mean=%f  sd=%f  best=%s=%f \n", mean, sdev, strat_name[best_strategy], estd[best_strategy]);
			      
		      	      if (! ((current_step<15 && best_payoff > mean + 1*sdev)
		      		     || current_step<25))
		      		  best_strategy = preferred_strategy;
		      	  }
		      else
			  // always play the preferred strategy (Sticky Sharp) for 25 moves.
			  if (current_step<25)
			      best_strategy = preferred_strategy;
		      // else
		      //  	  System.out.format("%d se %f\n ",current_step, (best_payoff-mean)/sdev);

		      preferred_strategy = best_strategy;  // prefer to stick with previous strategy.


		       // System.out.println("");  
		      // System.out.println(strat_name[best_strategy]);  
		  }
	  }
      else  // use Metacognition
	  {

      double sum[] = new double[NUM_STRATEGIES];
      double num[] = new double[NUM_STRATEGIES];
   

    for(int i=num_strategies; i>=0; i--)
	  {
	      sum[i] += (current_step<30 ?
			 initial_strategy_preference[i]
			 : deferred_strategy_preference[i]);
	  }

      int timeout = 0;
      for(int i=current_record; i>=0 && timeout++<1000; i--)
	  {
	      /* to do: add noise? */
	      int time_lapsed = current_step-hist_steps[i];
	      // double activation =  exp((4.0+log(pow(time_lapsed,-d)))/temperature)
	      double activation =  (4.0+log(pow(time_lapsed,-d)))/temperature
		  +  mRandom.nextFloat()*10 // this seems to improve things a bit
		  ;
	      
	      sum[hist_strategies[i]] += hist_payoffs [i] * activation;
	      // System.out.print("adding");
	      // System.out.println(hist_strategies[i]);
	      // System.out.println(activation);
	      // System.out.println(sum[hist_strategies[i]]);
	      num[hist_strategies[i]] += activation;
	  }

      sum[latest_strategy] += 0.05*num[latest_strategy];  // prefer last strategy


      /* pick best strategy */
     for(int i=num_strategies; i>=0; i--)
	  {
	      latest_actions[i] = -1;
	      if (meta_strategy==-1 || i==meta_strategy)
		  latest_actions[i] = strategies[i].getAction();
	      if (latest_actions[i] < 0)
		  latest_actions[i] = -1;  // the strategy doesn't know
	  }

 
      double best_payoff = -1;
      for(int i=num_strategies; i>=0; i--)
	  {
	      // System.out.print(i);
	      // System.out.print(": ");
	      // System.out.println(sum[i]);
	      if (sum[i]/num[i]>best_payoff && latest_actions[i]>-1)
	      {
		  best_payoff = sum[i]/num[i];
		  best_strategy = i;
	      }
	  }
	  }

      // System.out.print(current_step);
      // System.out.print(": ");
      // System.out.println(sum[best_strategy]);
 

      // System.out.println(best_strategy);
      latest_strategy = best_strategy;
      return latest_actions[best_strategy];
      //      return strategies[best_strategy].getAction();
  }
   
  
  
  /* Called at the end of each round returning the actions taken by the players,
     Stores the actions in three seperate integer arrays, and gets the utilities
     to be stored in seperate arrays for utilities */
  @Override
  public void observeOutcome(int[] actions) {
      /* i think we're [0] */

      if (USE_LMS || GATHER_STATS)
	  if (current_step<16) // do not update the metric after switching to final strategy.
	      super.observeOutcome(actions);


      if (current_step==99 && GATHER_STATS)
	  {

	      // print statistics
	      // c.1 s.1 p.1 c.2 s.2 p.2 meta.strategy meta.payoff
	      System.out.format("taylor %d %d %d %d %d %d %d %d\n",
				oppCooperationMetric[1],
				oppSandwichMetric[1],
				oppPredictabilityMetric[1],
				oppCooperationMetric[2],
				oppSandwichMetric[2],
				oppPredictabilityMetric[2],
				meta_strategy,
				total_payoff);

	  }

      if (LOG && current_step == 99)
	  System.out.format("final_strategy %s %d\n", strat_name[preferred_strategy], total_payoff);

      total_payoff += utility.getUtility(actions)[0];

      for(int i=num_strategies; i>=0; i--)
	  {

	      if (meta_strategy==-1 || i==latest_strategy)
		  {

		      int[] hypothetical_actions = {actions[0], 
						    actions[1],
						    actions[2]};

		      if (latest_actions[i] > -1)
			  {
			      // the hypothetical action is used when
			      // the strategy specified one
			      hypothetical_actions[0] = latest_actions[i];

			      // if (latest_strategy == i)  // hurts performance
			  
			      current_record++;
			      hist_steps[current_record] = current_step;
			      hist_strategies[current_record] = i; // redundant
			      int ut[] = utility.getUtility(hypothetical_actions);
			      hist_payoffs[current_record] = ut[0]/max(ut[1],ut[2]);  // min() hurts performance
		  
			  }
		      // let strategies observe the outcome (as if they had played)
		      if (observe_outcome_hypothetical[i]) //the third strategy needs real actions.
			  // Note: presenting a whole lot of hypothetical actions
			  // here won't help.
			  strategies[i].observeOutcome(hypothetical_actions);  // actions -> worse
		      else
			  strategies[i].observeOutcome(actions);  // actions -> worse
		  } 
	  }
  }



    void initLMs () {

	String modstring = 
	// just the first 15 moves
 "(Intercept)               7.116e+02  3.275e+01  21.724  < 2e-16 ***\nmeta.strategy4            8.238e+01  3.972e+01   2.074 0.038096 *  \nmeta.strategy5            1.532e+02  5.221e+01   2.934 0.003347 ** \nmeta.strategy7            1.163e+02  4.879e+01   2.383 0.017184 *  \nmeta.strategy11           1.357e+02  5.149e+01   2.635 0.008420 ** \nmeta.strategy12           1.458e+02  4.824e+01   3.022 0.002515 ** \nmeta.strategy13           1.814e+02  4.860e+01   3.732 0.000190 ***\nmeta.strategy0:ca         2.717e+00  6.954e-01   3.908 9.33e-05 ***\nmeta.strategy1:ca         2.813e+00  5.046e-01   5.576 2.48e-08 ***\nmeta.strategy2:ca         1.283e+00  6.247e-01   2.054 0.040024 *  \nmeta.strategy3:ca         1.637e+00  7.350e-01   2.227 0.025952 *  \nmeta.strategy6:ca         2.104e+00  5.299e-01   3.970 7.20e-05 ***\nmeta.strategy8:ca         2.542e+00  4.698e-01   5.410 6.34e-08 ***\nmeta.strategy9:ca         7.013e+00  5.132e-01  13.664  < 2e-16 ***\nmeta.strategy10:ca        4.435e+00  1.045e+00   4.245 2.19e-05 ***\nmeta.strategy0:sa         2.727e+00  1.113e+00   2.450 0.014300 *  \nmeta.strategy1:sa        -2.184e+00  6.547e-01  -3.336 0.000851 ***\nmeta.strategy4:sa        -2.125e+00  7.900e-01  -2.690 0.007137 ** \nmeta.strategy0:pa         1.380e+00  3.570e-01   3.865 0.000111 ***\nmeta.strategy1:pa         9.833e-01  2.200e-01   4.469 7.88e-06 ***\nmeta.strategy2:pa         9.891e-01  4.339e-01   2.280 0.022633 *  \nmeta.strategy3:pa         1.684e+00  4.338e-01   3.882 0.000104 ***\nmeta.strategy8:pa         1.062e+00  2.139e-01   4.963 6.95e-07 ***\nmeta.strategy9:pa         1.612e+00  1.997e-01   8.072 7.06e-16 ***\nmeta.strategy10:pa        8.900e-01  4.258e-01   2.090 0.036625 *  \nmeta.strategy0:ci        -1.325e+00  1.278e-01 -10.365  < 2e-16 ***\nmeta.strategy1:ci        -9.037e-01  8.619e-02 -10.484  < 2e-16 ***\nmeta.strategy4:ci        -5.570e-01  1.032e-01  -5.399 6.74e-08 ***\nmeta.strategy5:ci        -3.360e+00  8.926e-01  -3.764 0.000167 ***\nmeta.strategy6:ci        -9.545e-01  1.268e-01  -7.530 5.17e-14 ***\nmeta.strategy7:ci        -2.501e+00  5.364e-01  -4.662 3.14e-06 ***\nmeta.strategy8:ci        -4.116e-01  9.489e-02  -4.338 1.44e-05 ***\nmeta.strategy9:ci        -1.176e+00  8.754e-02 -13.438  < 2e-16 ***\nmeta.strategy10:ci       -2.444e+00  3.656e-01  -6.685 2.34e-11 ***\nmeta.strategy11:ci       -2.165e+00  8.612e-01  -2.514 0.011944 *  \nmeta.strategy13:ci       -2.265e+00  8.398e-01  -2.697 0.006991 ** \nmeta.strategy0:si        -1.678e+00  3.067e-01  -5.470 4.53e-08 ***\nmeta.strategy1:si         1.852e+00  2.407e-01   7.695 1.44e-14 ***\nmeta.strategy2:si         3.236e+00  7.828e-01   4.134 3.57e-05 ***\nmeta.strategy3:si         1.284e+00  5.614e-01   2.287 0.022192 *  \nmeta.strategy4:si         2.593e+00  2.654e-01   9.769  < 2e-16 ***\nmeta.strategy5:si        -2.217e+00  1.050e+00  -2.112 0.034688 *  \nmeta.strategy6:si         1.352e+00  3.157e-01   4.281 1.86e-05 ***\nmeta.strategy8:si         5.636e-01  2.383e-01   2.365 0.018038 *  \nmeta.strategy9:si         2.456e+00  2.308e-01  10.641  < 2e-16 ***\nmeta.strategy0:pi        -1.218e+00  1.006e-01 -12.109  < 2e-16 ***\nmeta.strategy1:pi        -7.338e-01  5.710e-02 -12.852  < 2e-16 ***\nmeta.strategy4:pi        -4.911e-01  7.307e-02  -6.722 1.82e-11 ***\nmeta.strategy5:pi        -2.649e+00  2.639e-01 -10.037  < 2e-16 ***\nmeta.strategy6:pi        -9.666e-01  8.702e-02 -11.107  < 2e-16 ***\nmeta.strategy7:pi        -1.238e+00  1.990e-01  -6.220 5.02e-10 ***\nmeta.strategy8:pi        -5.776e-01  6.738e-02  -8.572  < 2e-16 ***\nmeta.strategy9:pi        -1.160e+00  5.832e-02 -19.893  < 2e-16 ***\nmeta.strategy10:pi       -1.886e+00  1.929e-01  -9.776  < 2e-16 ***\nmeta.strategy11:pi       -2.773e+00  2.680e-01 -10.347  < 2e-16 ***\nmeta.strategy12:pi       -1.428e+00  2.402e-01  -5.945 2.78e-09 ***\nmeta.strategy13:pi       -2.695e+00  2.599e-01 -10.372  < 2e-16 ***\nmeta.strategy0:ca:sa     -1.339e-01  2.716e-02  -4.930 8.23e-07 ***\nmeta.strategy1:ca:sa     -6.253e-02  1.753e-02  -3.566 0.000362 ***\nmeta.strategy8:ca:sa     -1.181e-01  1.969e-02  -5.997 2.03e-09 ***\nmeta.strategy9:ca:sa     -1.002e-01  1.685e-02  -5.943 2.81e-09 ***\nmeta.strategy0:ca:pa     -3.123e-02  8.199e-03  -3.809 0.000140 ***\nmeta.strategy1:ca:pa     -3.606e-02  5.585e-03  -6.456 1.09e-10 ***\nmeta.strategy8:ca:pa     -3.274e-02  5.549e-03  -5.900 3.65e-09 ***\nmeta.strategy9:ca:pa     -7.644e-02  5.713e-03 -13.381  < 2e-16 ***\nmeta.strategy10:ca:pa    -3.295e-02  1.440e-02  -2.288 0.022148 *  \nmeta.strategy12:ca:pa     7.106e-02  3.578e-02   1.986 0.047002 *  \nmeta.strategy0:sa:pa     -5.594e-02  1.428e-02  -3.918 8.95e-05 ***\nmeta.strategy3:sa:pa     -4.905e-02  1.984e-02  -2.473 0.013421 *  \nmeta.strategy6:sa:pa      4.239e-02  1.115e-02   3.802 0.000144 ***\nmeta.strategy8:sa:pa     -3.982e-02  8.426e-03  -4.726 2.30e-06 ***\nmeta.strategy12:sa:pa     1.013e-01  3.388e-02   2.991 0.002778 ** \nmeta.strategy0:ci:si     -4.728e-02  1.223e-02  -3.864 0.000112 ***\nmeta.strategy2:ci:si      6.146e-02  1.623e-02   3.787 0.000153 ***\nmeta.strategy4:ci:si     -9.463e-02  1.929e-02  -4.905 9.38e-07 ***\nmeta.strategy6:ci:si      4.518e-02  1.867e-02   2.420 0.015543 *  \nmeta.strategy9:ci:si      4.718e-02  1.165e-02   4.049 5.16e-05 ***\nmeta.strategy0:ci:pi     -3.571e-02  2.946e-03 -12.121  < 2e-16 ***\nmeta.strategy1:ci:pi     -2.163e-02  1.794e-03 -12.057  < 2e-16 ***\nmeta.strategy2:ci:pi     -2.406e-02  3.436e-03  -7.002 2.57e-12 ***\nmeta.strategy3:ci:pi     -2.189e-02  3.185e-03  -6.872 6.42e-12 ***\nmeta.strategy4:ci:pi     -3.467e-02  2.846e-03 -12.182  < 2e-16 ***\nmeta.strategy6:ci:pi     -3.320e-02  3.102e-03 -10.703  < 2e-16 ***\nmeta.strategy7:ci:pi     -4.323e-02  9.404e-03  -4.597 4.29e-06 ***\nmeta.strategy8:ci:pi     -3.865e-02  2.345e-03 -16.485  < 2e-16 ***\nmeta.strategy9:ci:pi     -3.728e-02  1.729e-03 -21.560  < 2e-16 ***\nmeta.strategy1:si:pi      2.562e-02  5.080e-03   5.044 4.58e-07 ***\nmeta.strategy2:si:pi      3.555e-02  8.677e-03   4.097 4.19e-05 ***\nmeta.strategy3:si:pi      3.391e-02  1.045e-02   3.244 0.001179 ** \nmeta.strategy4:si:pi      1.654e-02  6.251e-03   2.645 0.008163 ** \nmeta.strategy5:si:pi      8.198e-02  3.494e-02   2.346 0.018969 *  \nmeta.strategy6:si:pi      6.270e-02  6.577e-03   9.534  < 2e-16 ***\nmeta.strategy7:si:pi      8.568e-02  1.610e-02   5.321 1.04e-07 ***\nmeta.strategy8:si:pi     -1.225e-02  4.702e-03  -2.605 0.009181 ** \nmeta.strategy9:si:pi      3.656e-02  4.661e-03   7.843 4.51e-15 ***\nmeta.strategy10:si:pi     1.018e-01  1.519e-02   6.704 2.05e-11 ***\nmeta.strategy11:si:pi     1.053e-01  3.003e-02   3.507 0.000453 ***\nmeta.strategy12:si:pi     1.452e-01  2.480e-02   5.854 4.83e-09 ***\nmeta.strategy0:cs:ss      4.150e-02  5.702e-03   7.277 3.47e-13 ***\nmeta.strategy1:cs:ss      2.126e-02  4.197e-03   5.066 4.08e-07 ***\nmeta.strategy3:cs:ss      1.957e-02  8.250e-03   2.372 0.017716 *  \nmeta.strategy4:cs:ss      2.251e-02  8.522e-03   2.642 0.008248 ** \nmeta.strategy6:cs:ss     -2.080e-02  8.252e-03  -2.520 0.011725 *  \nmeta.strategy8:cs:ss      2.892e-02  6.146e-03   4.705 2.55e-06 ***\nmeta.strategy9:cs:ss     -1.160e-02  4.077e-03  -2.846 0.004427 ** \nmeta.strategy0:cs:ps      1.049e-02  1.643e-03   6.387 1.71e-10 ***\nmeta.strategy1:cs:ps      9.818e-03  8.975e-04  10.939  < 2e-16 ***\nmeta.strategy4:cs:ps      5.491e-03  1.558e-03   3.524 0.000426 ***\nmeta.strategy6:cs:ps      5.453e-03  1.685e-03   3.236 0.001212 ** \nmeta.strategy7:cs:ps      1.663e-02  5.796e-03   2.870 0.004111 ** \nmeta.strategy8:cs:ps      8.532e-03  1.196e-03   7.132 1.00e-12 ***\nmeta.strategy9:cs:ps      1.075e-02  9.312e-04  11.539  < 2e-16 ***\nmeta.strategy0:ss:ps      2.157e-02  3.442e-03   6.267 3.73e-10 ***\nmeta.strategy1:ss:ps      5.873e-03  2.428e-03   2.419 0.015550 *  \nmeta.strategy6:ss:ps     -2.724e-02  3.682e-03  -7.399 1.39e-13 ***\nmeta.strategy8:ss:ps      1.317e-02  2.168e-03   6.071 1.28e-09 ***\nmeta.strategy9:ss:ps     -2.022e-02  2.143e-03  -9.437  < 2e-16 ***\nmeta.strategy10:ss:ps    -2.329e-02  8.699e-03  -2.678 0.007418 ** \nmeta.strategy12:ss:ps    -4.750e-02  1.354e-02  -3.508 0.000453 ***\nmeta.strategy0:ca:sa:pa   7.122e-04  2.809e-04   2.536 0.011221 *  \nmeta.strategy1:ca:sa:pa   6.479e-04  1.704e-04   3.802 0.000143 ***\nmeta.strategy6:ca:sa:pa  -5.498e-04  2.101e-04  -2.617 0.008876 ** \nmeta.strategy8:ca:sa:pa   1.050e-03  1.848e-04   5.680 1.36e-08 ***\nmeta.strategy9:ca:sa:pa   9.064e-04  1.695e-04   5.347 8.99e-08 ***\nmeta.strategy12:ca:sa:pa -2.990e-03  1.176e-03  -2.541 0.011049 *  \nmeta.strategy0:ci:si:pi  -5.373e-04  1.504e-04  -3.573 0.000353 ***\nmeta.strategy1:ci:si:pi  -8.256e-04  1.165e-04  -7.087 1.39e-12 ***\nmeta.strategy2:ci:si:pi  -4.921e-04  1.883e-04  -2.613 0.008983 ** \nmeta.strategy4:ci:si:pi   6.554e-04  1.597e-04   4.104 4.07e-05 ***\nmeta.strategy6:ci:si:pi  -7.904e-04  1.646e-04  -4.801 1.58e-06 ***\nmeta.strategy7:ci:si:pi  -9.727e-04  3.325e-04  -2.926 0.003438 ** \nmeta.strategy8:ci:si:pi  -6.379e-04  1.254e-04  -5.085 3.68e-07 ***\nmeta.strategy10:ci:si:pi -1.726e-03  3.271e-04  -5.277 1.32e-07 ***\nmeta.strategy12:ci:si:pi -2.887e-03  8.524e-04  -3.387 0.000707 ***\nmeta.strategy0:cs:ss:ps  -7.405e-05  3.021e-05  -2.451 0.014231 *  \nmeta.strategy1:cs:ss:ps  -1.547e-04  1.813e-05  -8.533  < 2e-16 ***\nmeta.strategy3:cs:ss:ps  -1.752e-04  4.287e-05  -4.086 4.39e-05 ***\nmeta.strategy6:cs:ss:ps   2.174e-04  2.949e-05   7.373 1.70e-13 ***\nmeta.strategy8:cs:ss:ps  -8.523e-05  2.254e-05  -3.782 0.000156 ***\nmeta.strategy9:cs:ss:ps   1.547e-04  1.768e-05   8.753  < 2e-16 ***\nmeta.strategy10:cs:ss:ps  2.300e-04  7.745e-05   2.969 0.002985 ** \nmeta.strategy12:cs:ss:ps  6.302e-04  2.238e-04   2.817 0.004856 ** \nSignif. codes:  0 ‘***’ 0.001 ‘**’ 0.01 ‘*’ 0.05 ‘.’ 0.1 ‘ ’ 1 \n(Intercept)            7.661e+02  2.463e+01  31.101  < 2e-16 ***\nmeta.strategy4         6.648e+01  3.048e+01   2.181 0.029162 *  \nmeta.strategy12        1.169e+02  3.523e+01   3.319 0.000905 ***\nmeta.strategy13        9.098e+01  3.692e+01   2.464 0.013740 *  \nmeta.strategy3:ca      1.087e+00  5.288e-01   2.056 0.039743 *  \nmeta.strategy6:ca      1.140e+00  3.519e-01   3.238 0.001203 ** \nmeta.strategy7:ca      2.352e+00  9.439e-01   2.492 0.012714 *  \nmeta.strategy8:ca     -1.589e+00  2.587e-01  -6.140 8.31e-10 ***\nmeta.strategy11:ca    -3.715e+00  1.581e+00  -2.350 0.018779 *  \nmeta.strategy13:ca    -4.241e+00  1.591e+00  -2.666 0.007676 ** \nmeta.strategy1:sa     -2.188e+00  4.324e-01  -5.059 4.22e-07 ***\nmeta.strategy2:sa      1.591e+00  6.759e-01   2.354 0.018582 *  \nmeta.strategy9:sa     -2.494e+00  4.314e-01  -5.782 7.43e-09 ***\nmeta.strategy10:sa    -2.008e+00  9.581e-01  -2.096 0.036077 *  \nmeta.strategy12:sa    -3.593e+00  1.599e+00  -2.247 0.024662 *  \nmeta.strategy0:pa      9.778e-01  3.956e-01   2.471 0.013462 *  \nmeta.strategy3:pa      1.080e+00  4.612e-01   2.341 0.019244 *  \nmeta.strategy5:pa      1.224e+00  4.229e-01   2.894 0.003805 ** \nmeta.strategy7:pa      1.259e+00  3.812e-01   3.301 0.000964 ***\nmeta.strategy9:pa      1.110e+00  3.602e-01   3.081 0.002068 ** \nmeta.strategy10:pa     1.570e+00  3.788e-01   4.146 3.39e-05 ***\nmeta.strategy11:pa     1.001e+00  4.091e-01   2.447 0.014408 *  \nmeta.strategy1:ci      4.560e+00  4.959e-01   9.195  < 2e-16 ***\nmeta.strategy3:ci      1.271e+00  5.751e-01   2.211 0.027060 *  \nmeta.strategy4:ci     -1.799e+00  5.075e-01  -3.546 0.000392 ***\nmeta.strategy7:ci     -3.044e+00  9.649e-01  -3.155 0.001605 ** \nmeta.strategy8:ci      4.609e+00  4.638e-01   9.937  < 2e-16 ***\nmeta.strategy9:ci      4.866e+00  5.441e-01   8.944  < 2e-16 ***\nmeta.strategy0:si     -2.584e+00  1.017e+00  -2.539 0.011113 *  \nmeta.strategy2:si     -2.809e+00  8.090e-01  -3.472 0.000517 ***\nmeta.strategy4:si     -2.043e+00  7.481e-01  -2.731 0.006320 ** \nmeta.strategy8:si     -1.956e+00  6.710e-01  -2.915 0.003555 ** \nmeta.strategy1:pi      1.162e+00  4.047e-01   2.871 0.004096 ** \nmeta.strategy2:pi     -1.505e+00  4.204e-01  -3.579 0.000346 ***\nmeta.strategy3:pi     -9.974e-01  4.386e-01  -2.274 0.022964 *  \nmeta.strategy4:pi     -9.800e-01  3.677e-01  -2.665 0.007693 ** \nmeta.strategy5:pi     -1.466e+00  4.367e-01  -3.357 0.000788 ***\nmeta.strategy7:pi     -1.399e+00  4.117e-01  -3.399 0.000677 ***\nmeta.strategy9:pi      1.285e+00  4.350e-01   2.953 0.003144 ** \nmeta.strategy11:pi    -1.337e+00  4.425e-01  -3.022 0.002514 ** \nmeta.strategy13:pi    -1.281e+00  4.300e-01  -2.980 0.002888 ** \nmeta.strategy0:ca:sa  -3.452e-02  2.105e-03 -16.399  < 2e-16 ***\nmeta.strategy2:ca:sa  -2.924e-02  2.872e-03 -10.182  < 2e-16 ***\nmeta.strategy5:ca:sa  -1.510e-01  6.149e-02  -2.456 0.014052 *  \nmeta.strategy10:ca:sa -2.904e-02  1.117e-02  -2.601 0.009306 ** \nmeta.strategy1:ca:pa  -3.443e-02  4.201e-03  -8.196 2.56e-16 ***\nmeta.strategy6:ca:pa   2.317e-02  4.486e-03   5.166 2.41e-07 ***\nmeta.strategy7:ca:pa  -2.140e-02  9.751e-03  -2.195 0.028188 *  \nmeta.strategy8:ca:pa  -7.509e-03  3.173e-03  -2.366 0.017964 *  \nmeta.strategy9:ca:pa  -4.679e-02  4.823e-03  -9.702  < 2e-16 ***\nmeta.strategy0:ca:ci   3.621e-02  7.931e-03   4.566 4.98e-06 ***\nmeta.strategy1:ca:ci  -9.325e-02  7.707e-03 -12.099  < 2e-16 ***\nmeta.strategy4:ca:ci   3.834e-02  8.248e-03   4.649 3.35e-06 ***\nmeta.strategy5:ca:ci  -4.957e-01  1.874e-01  -2.646 0.008153 ** \nmeta.strategy8:ca:ci  -3.663e-02  7.086e-03  -5.170 2.35e-07 ***\nmeta.strategy9:ca:ci  -1.478e-01  8.692e-03 -17.002  < 2e-16 ***\nmeta.strategy11:ca:ci -5.828e-01  1.706e-01  -3.416 0.000637 ***\nmeta.strategy12:ca:ci -3.388e-01  1.623e-01  -2.088 0.036845 *  \nmeta.strategy13:ca:ci -5.093e-01  1.649e-01  -3.089 0.002012 ** \nmeta.strategy0:ca:si   3.253e-02  2.768e-03  11.751  < 2e-16 ***\nmeta.strategy2:ca:si   1.922e-02  4.003e-03   4.801 1.59e-06 ***\nmeta.strategy5:ca:si   1.739e-01  6.380e-02   2.725 0.006433 ** \nmeta.strategy6:ca:si   3.771e-02  6.553e-03   5.755 8.74e-09 ***\nmeta.strategy8:ca:si   1.449e-02  4.202e-03   3.449 0.000564 ***\nmeta.strategy9:ca:si   1.515e-02  2.540e-03   5.965 2.47e-09 ***\nmeta.strategy0:ca:pi   7.312e-03  1.024e-03   7.138 9.58e-13 ***\nmeta.strategy1:ca:pi   6.113e-03  7.923e-04   7.716 1.23e-14 ***\nmeta.strategy3:ca:pi  -1.556e-02  1.932e-03  -8.055 8.11e-16 ***\nmeta.strategy4:ca:pi   6.196e-03  1.242e-03   4.988 6.12e-07 ***\nmeta.strategy5:ca:pi   3.273e-02  1.109e-02   2.952 0.003160 ** \nmeta.strategy6:ca:pi   1.427e-02  1.294e-03  11.030  < 2e-16 ***\nmeta.strategy7:ca:pi   1.736e-02  5.459e-03   3.180 0.001472 ** \nmeta.strategy8:ca:pi   9.038e-03  9.413e-04   9.601  < 2e-16 ***\nmeta.strategy9:ca:pi   1.651e-02  8.369e-04  19.725  < 2e-16 ***\nmeta.strategy10:ca:pi  1.033e-02  3.007e-03   3.434 0.000595 ***\nmeta.strategy11:ca:pi  3.320e-02  1.149e-02   2.890 0.003858 ** \nmeta.strategy0:ca:cs   6.053e-03  2.618e-03   2.313 0.020744 *  \nmeta.strategy1:ca:cs   3.970e-02  2.741e-03  14.485  < 2e-16 ***\nmeta.strategy2:ca:cs   8.613e-03  3.586e-03   2.401 0.016336 *  \nmeta.strategy5:ca:cs   1.215e-01  4.831e-02   2.516 0.011882 *  \nmeta.strategy6:ca:cs  -2.754e-02  2.936e-03  -9.379  < 2e-16 ***\nmeta.strategy8:ca:cs   2.497e-02  2.389e-03  10.452  < 2e-16 ***\nmeta.strategy9:ca:cs   5.815e-02  2.941e-03  19.776  < 2e-16 ***\nmeta.strategy10:ca:cs -1.234e-02  4.107e-03  -3.005 0.002659 ** \nmeta.strategy11:ca:cs  1.469e-01  4.414e-02   3.328 0.000875 ***\nmeta.strategy13:ca:cs  1.397e-01  4.260e-02   3.279 0.001041 ** \nmeta.strategy1:sa:pa   1.431e-02  4.645e-03   3.080 0.002074 ** \nmeta.strategy9:sa:pa   3.420e-02  4.863e-03   7.032 2.06e-12 ***\nmeta.strategy1:sa:ci  -2.436e-02  3.348e-03  -7.276 3.50e-13 ***\nmeta.strategy2:sa:ci  -4.479e-02  4.236e-03 -10.574  < 2e-16 ***\nmeta.strategy3:sa:ci  -1.067e-02  5.246e-03  -2.034 0.041913 *  \nmeta.strategy5:sa:ci   1.314e-01  6.160e-02   2.134 0.032880 *  \nmeta.strategy6:sa:ci   2.952e-02  5.709e-03   5.171 2.34e-07 ***\nmeta.strategy7:sa:ci   2.556e-02  6.995e-03   3.654 0.000259 ***\nmeta.strategy8:sa:ci  -2.049e-02  4.662e-03  -4.394 1.11e-05 ***\nmeta.strategy9:sa:ci   1.648e-02  1.788e-03   9.219  < 2e-16 ***\nmeta.strategy10:sa:ci  4.016e-02  8.888e-03   4.518 6.25e-06 ***\nmeta.strategy1:sa:si  -4.317e-02  1.227e-02  -3.518 0.000436 ***\nmeta.strategy4:sa:si  -1.538e-01  1.749e-02  -8.794  < 2e-16 ***\nmeta.strategy6:sa:si   1.058e-01  2.156e-02   4.907 9.27e-07 ***\nmeta.strategy7:sa:si   7.024e-02  2.644e-02   2.656 0.007900 ** \nmeta.strategy8:sa:si  -6.405e-02  1.602e-02  -3.998 6.39e-05 ***\nmeta.strategy9:sa:si   7.333e-02  1.312e-02   5.591 2.27e-08 ***\nmeta.strategy10:sa:si  9.392e-02  3.428e-02   2.740 0.006152 ** \nmeta.strategy12:sa:si  3.417e-01  1.478e-01   2.312 0.020763 *  \nmeta.strategy1:sa:pi  -1.096e-02  1.904e-03  -5.758 8.57e-09 ***\nmeta.strategy2:sa:pi  -2.521e-02  2.699e-03  -9.338  < 2e-16 ***\nmeta.strategy3:sa:pi  -2.065e-02  4.157e-03  -4.969 6.76e-07 ***\nmeta.strategy4:sa:pi  -1.448e-02  2.985e-03  -4.850 1.24e-06 ***\nmeta.strategy7:sa:pi  -1.256e-02  6.074e-03  -2.068 0.038621 *  \nmeta.strategy8:sa:pi   7.695e-03  2.205e-03   3.490 0.000483 ***\nmeta.strategy3:sa:ss   1.157e-02  5.554e-03   2.083 0.037239 *  \nmeta.strategy6:sa:ss  -1.859e-02  3.918e-03  -4.745 2.09e-06 ***\nmeta.strategy8:sa:ss  -8.285e-03  3.474e-03  -2.385 0.017088 *  \nmeta.strategy9:sa:ss  -3.633e-02  2.785e-03 -13.045  < 2e-16 ***\nmeta.strategy6:pa:ci  -1.948e-02  6.848e-03  -2.845 0.004446 ** \nmeta.strategy7:pa:ci   4.372e-02  1.032e-02   4.234 2.30e-05 ***\nmeta.strategy8:pa:ci  -2.965e-02  5.597e-03  -5.296 1.19e-07 ***\nmeta.strategy1:pa:si   2.394e-02  6.625e-03   3.613 0.000303 ***\nmeta.strategy4:pa:si   3.212e-02  9.128e-03   3.519 0.000434 ***\nmeta.strategy6:pa:si  -3.403e-02  1.160e-02  -2.933 0.003358 ** \nmeta.strategy8:pa:si   3.227e-02  7.779e-03   4.148 3.36e-05 ***\nmeta.strategy9:pa:si  -2.056e-02  7.448e-03  -2.761 0.005771 ** \nmeta.strategy12:pa:si -2.886e-02  1.366e-02  -2.113 0.034620 *  \nmeta.strategy0:pa:pi   2.113e-02  7.225e-03   2.924 0.003458 ** \nmeta.strategy1:pa:pi  -2.742e-02  6.387e-03  -4.294 1.76e-05 ***\nmeta.strategy2:pa:pi   3.558e-02  9.625e-03   3.697 0.000219 ***\nmeta.strategy3:pa:pi   2.837e-02  7.350e-03   3.860 0.000114 ***\nmeta.strategy5:pa:pi   2.279e-02  7.741e-03   2.944 0.003245 ** \nmeta.strategy9:pa:pi  -2.686e-02  7.044e-03  -3.814 0.000137 ***\nmeta.strategy12:pa:pi -1.870e-02  7.820e-03  -2.391 0.016787 *  \nmeta.strategy1:pa:ps   6.346e-03  2.352e-03   2.698 0.006974 ** \nmeta.strategy3:pa:ps  -1.004e-02  3.730e-03  -2.692 0.007105 ** \nmeta.strategy5:pa:ps  -6.350e-03  2.049e-03  -3.099 0.001944 ** \nmeta.strategy7:pa:ps  -7.189e-03  2.188e-03  -3.285 0.001020 ** \nmeta.strategy10:pa:ps -1.001e-02  2.408e-03  -4.156 3.24e-05 ***\nmeta.strategy13:pa:ps -5.785e-03  1.978e-03  -2.925 0.003446 ** \nmeta.strategy2:ci:si   4.382e-02  6.689e-03   6.551 5.78e-11 ***\nmeta.strategy4:ci:si   2.727e-02  7.876e-03   3.463 0.000535 ***\nmeta.strategy5:ci:si  -1.612e-01  6.423e-02  -2.509 0.012108 *  \nmeta.strategy6:ci:si  -3.665e-02  8.756e-03  -4.185 2.85e-05 ***\nmeta.strategy7:ci:si  -8.141e-02  1.565e-02  -5.203 1.97e-07 ***\nmeta.strategy9:ci:si   2.239e-02  5.856e-03   3.823 0.000132 ***\nmeta.strategy10:ci:si -3.185e-02  1.204e-02  -2.646 0.008152 ** \nmeta.strategy0:ci:pi  -2.856e-02  1.268e-03 -22.519  < 2e-16 ***\nmeta.strategy1:ci:pi  -1.572e-02  1.174e-03 -13.393  < 2e-16 ***\nmeta.strategy2:ci:pi  -2.728e-02  1.285e-03 -21.229  < 2e-16 ***\nmeta.strategy3:ci:pi  -2.266e-02  1.051e-03 -21.558  < 2e-16 ***\nmeta.strategy4:ci:pi  -3.046e-02  1.321e-03 -23.052  < 2e-16 ***\nmeta.strategy5:ci:pi  -3.528e-02  1.134e-02  -3.111 0.001865 ** \nmeta.strategy6:ci:pi  -3.331e-02  1.446e-03 -23.038  < 2e-16 ***\nmeta.strategy7:ci:pi  -3.581e-02  5.052e-03  -7.088 1.38e-12 ***\nmeta.strategy8:ci:pi  -3.364e-02  1.211e-03 -27.783  < 2e-16 ***\nmeta.strategy9:ci:pi  -2.832e-02  9.973e-04 -28.394  < 2e-16 ***\nmeta.strategy10:ci:pi -2.120e-02  3.724e-03  -5.694 1.25e-08 ***\nmeta.strategy11:ci:pi -3.719e-02  1.148e-02  -3.239 0.001199 ** \nmeta.strategy12:ci:pi -3.845e-02  1.115e-02  -3.447 0.000567 ***\nmeta.strategy13:ci:pi -2.345e-02  1.113e-02  -2.107 0.035136 *  \nmeta.strategy0:ci:cs  -1.724e-02  2.599e-03  -6.632 3.34e-11 ***\nmeta.strategy2:ci:cs  -6.816e-03  2.296e-03  -2.968 0.002997 ** \nmeta.strategy4:ci:cs  -1.048e-02  3.015e-03  -3.476 0.000510 ***\nmeta.strategy5:ci:cs   1.292e-01  4.617e-02   2.799 0.005128 ** \nmeta.strategy6:ci:cs   1.618e-02  3.097e-03   5.225 1.75e-07 ***\nmeta.strategy9:ci:cs   1.757e-02  2.612e-03   6.726 1.76e-11 ***\nmeta.strategy11:ci:cs  1.455e-01  4.210e-02   3.457 0.000547 ***\nmeta.strategy12:ci:cs  9.343e-02  3.978e-02   2.349 0.018836 *  \nmeta.strategy13:ci:cs  1.176e-01  4.071e-02   2.889 0.003864 ** \nmeta.strategy0:si:pi  -1.377e-02  4.490e-03  -3.067 0.002162 ** \nmeta.strategy2:si:pi   1.702e-02  4.145e-03   4.105 4.04e-05 ***\nmeta.strategy3:si:pi   3.236e-02  5.958e-03   5.432 5.61e-08 ***\nmeta.strategy4:si:pi   1.146e-02  4.724e-03   2.427 0.015222 *  \nmeta.strategy5:si:pi   5.215e-02  1.477e-02   3.531 0.000414 ***\nmeta.strategy6:si:pi   1.535e-02  4.732e-03   3.243 0.001182 ** \nmeta.strategy7:si:pi   5.388e-02  6.877e-03   7.835 4.79e-15 ***\nmeta.strategy8:si:pi  -3.825e-02  3.322e-03 -11.516  < 2e-16 ***\nmeta.strategy9:si:pi  -1.391e-02  3.549e-03  -3.919 8.90e-05 ***\nmeta.strategy10:si:pi  3.750e-02  6.603e-03   5.679 1.37e-08 ***\nmeta.strategy11:si:pi  4.834e-02  1.278e-02   3.783 0.000155 ***\nmeta.strategy12:si:pi  4.425e-02  1.136e-02   3.894 9.86e-05 ***\nmeta.strategy13:si:pi  4.169e-02  1.438e-02   2.900 0.003736 ** \nmeta.strategy0:si:ss   4.767e-02  8.871e-03   5.374 7.72e-08 ***\nmeta.strategy1:si:ss   3.535e-02  6.599e-03   5.357 8.51e-08 ***\nmeta.strategy4:si:ss   8.624e-02  8.011e-03  10.766  < 2e-16 ***\nmeta.strategy6:si:ss  -3.511e-02  1.014e-02  -3.463 0.000535 ***\nmeta.strategy7:si:ss  -3.013e-02  1.384e-02  -2.177 0.029516 *  \nmeta.strategy8:si:ss   5.233e-02  6.675e-03   7.840 4.61e-15 ***\nmeta.strategy12:si:ss -8.916e-02  4.333e-02  -2.058 0.039615 *  \nmeta.strategy0:pi:ps  -1.032e-02  1.839e-03  -5.612 2.01e-08 ***\nmeta.strategy2:pi:ps  -6.305e-03  1.822e-03  -3.461 0.000539 ***\nmeta.strategy3:pi:ps   3.846e-03  1.692e-03   2.273 0.023060 *  \nmeta.strategy4:pi:ps   4.685e-03  1.846e-03   2.538 0.011165 *  \nmeta.strategy5:pi:ps  -1.206e-02  2.394e-03  -5.040 4.68e-07 ***\nmeta.strategy6:pi:ps  -4.658e-03  2.002e-03  -2.326 0.019997 *  \nmeta.strategy7:pi:ps   8.115e-03  2.128e-03   3.814 0.000137 ***\nmeta.strategy8:pi:ps  -8.383e-03  1.602e-03  -5.233 1.68e-07 ***\nmeta.strategy11:pi:ps -9.500e-03  2.319e-03  -4.097 4.20e-05 ***\nmeta.strategy13:pi:ps -8.255e-03  2.407e-03  -3.429 0.000606 ***\n";

	// smaller model produced just from the first 20
	modstring="(Intercept)               7.116e+02  3.275e+01  21.724  < 2e-16 ***\nmeta.strategy4            8.238e+01  3.972e+01   2.074 0.038096 *  \nmeta.strategy5            1.532e+02  5.221e+01   2.934 0.003347 ** \nmeta.strategy7            1.163e+02  4.879e+01   2.383 0.017184 *  \nmeta.strategy11           1.357e+02  5.149e+01   2.635 0.008420 ** \nmeta.strategy12           1.458e+02  4.824e+01   3.022 0.002515 ** \nmeta.strategy13           1.814e+02  4.860e+01   3.732 0.000190 ***\nmeta.strategy0:ca         2.717e+00  6.954e-01   3.908 9.33e-05 ***\nmeta.strategy1:ca         2.813e+00  5.046e-01   5.576 2.48e-08 ***\nmeta.strategy2:ca         1.283e+00  6.247e-01   2.054 0.040024 *  \nmeta.strategy3:ca         1.637e+00  7.350e-01   2.227 0.025952 *  \nmeta.strategy6:ca         2.104e+00  5.299e-01   3.970 7.20e-05 ***\nmeta.strategy8:ca         2.542e+00  4.698e-01   5.410 6.34e-08 ***\nmeta.strategy9:ca         7.013e+00  5.132e-01  13.664  < 2e-16 ***\nmeta.strategy10:ca        4.435e+00  1.045e+00   4.245 2.19e-05 ***\nmeta.strategy0:sa         2.727e+00  1.113e+00   2.450 0.014300 *  \nmeta.strategy1:sa        -2.184e+00  6.547e-01  -3.336 0.000851 ***\nmeta.strategy4:sa        -2.125e+00  7.900e-01  -2.690 0.007137 ** \nmeta.strategy0:pa         1.380e+00  3.570e-01   3.865 0.000111 ***\nmeta.strategy1:pa         9.833e-01  2.200e-01   4.469 7.88e-06 ***\nmeta.strategy2:pa         9.891e-01  4.339e-01   2.280 0.022633 *  \nmeta.strategy3:pa         1.684e+00  4.338e-01   3.882 0.000104 ***\nmeta.strategy8:pa         1.062e+00  2.139e-01   4.963 6.95e-07 ***\nmeta.strategy9:pa         1.612e+00  1.997e-01   8.072 7.06e-16 ***\nmeta.strategy10:pa        8.900e-01  4.258e-01   2.090 0.036625 *  \nmeta.strategy0:ci        -1.325e+00  1.278e-01 -10.365  < 2e-16 ***\nmeta.strategy1:ci        -9.037e-01  8.619e-02 -10.484  < 2e-16 ***\nmeta.strategy4:ci        -5.570e-01  1.032e-01  -5.399 6.74e-08 ***\nmeta.strategy5:ci        -3.360e+00  8.926e-01  -3.764 0.000167 ***\nmeta.strategy6:ci        -9.545e-01  1.268e-01  -7.530 5.17e-14 ***\nmeta.strategy7:ci        -2.501e+00  5.364e-01  -4.662 3.14e-06 ***\nmeta.strategy8:ci        -4.116e-01  9.489e-02  -4.338 1.44e-05 ***\nmeta.strategy9:ci        -1.176e+00  8.754e-02 -13.438  < 2e-16 ***\nmeta.strategy10:ci       -2.444e+00  3.656e-01  -6.685 2.34e-11 ***\nmeta.strategy11:ci       -2.165e+00  8.612e-01  -2.514 0.011944 *  \nmeta.strategy13:ci       -2.265e+00  8.398e-01  -2.697 0.006991 ** \nmeta.strategy0:si        -1.678e+00  3.067e-01  -5.470 4.53e-08 ***\nmeta.strategy1:si         1.852e+00  2.407e-01   7.695 1.44e-14 ***\nmeta.strategy2:si         3.236e+00  7.828e-01   4.134 3.57e-05 ***\nmeta.strategy3:si         1.284e+00  5.614e-01   2.287 0.022192 *  \nmeta.strategy4:si         2.593e+00  2.654e-01   9.769  < 2e-16 ***\nmeta.strategy5:si        -2.217e+00  1.050e+00  -2.112 0.034688 *  \nmeta.strategy6:si         1.352e+00  3.157e-01   4.281 1.86e-05 ***\nmeta.strategy8:si         5.636e-01  2.383e-01   2.365 0.018038 *  \nmeta.strategy9:si         2.456e+00  2.308e-01  10.641  < 2e-16 ***\nmeta.strategy0:pi        -1.218e+00  1.006e-01 -12.109  < 2e-16 ***\nmeta.strategy1:pi        -7.338e-01  5.710e-02 -12.852  < 2e-16 ***\nmeta.strategy4:pi        -4.911e-01  7.307e-02  -6.722 1.82e-11 ***\nmeta.strategy5:pi        -2.649e+00  2.639e-01 -10.037  < 2e-16 ***\nmeta.strategy6:pi        -9.666e-01  8.702e-02 -11.107  < 2e-16 ***\nmeta.strategy7:pi        -1.238e+00  1.990e-01  -6.220 5.02e-10 ***\nmeta.strategy8:pi        -5.776e-01  6.738e-02  -8.572  < 2e-16 ***\nmeta.strategy9:pi        -1.160e+00  5.832e-02 -19.893  < 2e-16 ***\nmeta.strategy10:pi       -1.886e+00  1.929e-01  -9.776  < 2e-16 ***\nmeta.strategy11:pi       -2.773e+00  2.680e-01 -10.347  < 2e-16 ***\nmeta.strategy12:pi       -1.428e+00  2.402e-01  -5.945 2.78e-09 ***\nmeta.strategy13:pi       -2.695e+00  2.599e-01 -10.372  < 2e-16 ***\nmeta.strategy0:ca:sa     -1.339e-01  2.716e-02  -4.930 8.23e-07 ***\nmeta.strategy1:ca:sa     -6.253e-02  1.753e-02  -3.566 0.000362 ***\nmeta.strategy8:ca:sa     -1.181e-01  1.969e-02  -5.997 2.03e-09 ***\nmeta.strategy9:ca:sa     -1.002e-01  1.685e-02  -5.943 2.81e-09 ***\nmeta.strategy0:ca:pa     -3.123e-02  8.199e-03  -3.809 0.000140 ***\nmeta.strategy1:ca:pa     -3.606e-02  5.585e-03  -6.456 1.09e-10 ***\nmeta.strategy8:ca:pa     -3.274e-02  5.549e-03  -5.900 3.65e-09 ***\nmeta.strategy9:ca:pa     -7.644e-02  5.713e-03 -13.381  < 2e-16 ***\nmeta.strategy10:ca:pa    -3.295e-02  1.440e-02  -2.288 0.022148 *  \nmeta.strategy12:ca:pa     7.106e-02  3.578e-02   1.986 0.047002 *  \nmeta.strategy0:sa:pa     -5.594e-02  1.428e-02  -3.918 8.95e-05 ***\nmeta.strategy3:sa:pa     -4.905e-02  1.984e-02  -2.473 0.013421 *  \nmeta.strategy6:sa:pa      4.239e-02  1.115e-02   3.802 0.000144 ***\nmeta.strategy8:sa:pa     -3.982e-02  8.426e-03  -4.726 2.30e-06 ***\nmeta.strategy12:sa:pa     1.013e-01  3.388e-02   2.991 0.002778 ** \nmeta.strategy0:ci:si     -4.728e-02  1.223e-02  -3.864 0.000112 ***\nmeta.strategy2:ci:si      6.146e-02  1.623e-02   3.787 0.000153 ***\nmeta.strategy4:ci:si     -9.463e-02  1.929e-02  -4.905 9.38e-07 ***\nmeta.strategy6:ci:si      4.518e-02  1.867e-02   2.420 0.015543 *  \nmeta.strategy9:ci:si      4.718e-02  1.165e-02   4.049 5.16e-05 ***\nmeta.strategy0:ci:pi     -3.571e-02  2.946e-03 -12.121  < 2e-16 ***\nmeta.strategy1:ci:pi     -2.163e-02  1.794e-03 -12.057  < 2e-16 ***\nmeta.strategy2:ci:pi     -2.406e-02  3.436e-03  -7.002 2.57e-12 ***\nmeta.strategy3:ci:pi     -2.189e-02  3.185e-03  -6.872 6.42e-12 ***\nmeta.strategy4:ci:pi     -3.467e-02  2.846e-03 -12.182  < 2e-16 ***\nmeta.strategy6:ci:pi     -3.320e-02  3.102e-03 -10.703  < 2e-16 ***\nmeta.strategy7:ci:pi     -4.323e-02  9.404e-03  -4.597 4.29e-06 ***\nmeta.strategy8:ci:pi     -3.865e-02  2.345e-03 -16.485  < 2e-16 ***\nmeta.strategy9:ci:pi     -3.728e-02  1.729e-03 -21.560  < 2e-16 ***\nmeta.strategy1:si:pi      2.562e-02  5.080e-03   5.044 4.58e-07 ***\nmeta.strategy2:si:pi      3.555e-02  8.677e-03   4.097 4.19e-05 ***\nmeta.strategy3:si:pi      3.391e-02  1.045e-02   3.244 0.001179 ** \nmeta.strategy4:si:pi      1.654e-02  6.251e-03   2.645 0.008163 ** \nmeta.strategy5:si:pi      8.198e-02  3.494e-02   2.346 0.018969 *  \nmeta.strategy6:si:pi      6.270e-02  6.577e-03   9.534  < 2e-16 ***\nmeta.strategy7:si:pi      8.568e-02  1.610e-02   5.321 1.04e-07 ***\nmeta.strategy8:si:pi     -1.225e-02  4.702e-03  -2.605 0.009181 ** \nmeta.strategy9:si:pi      3.656e-02  4.661e-03   7.843 4.51e-15 ***\nmeta.strategy10:si:pi     1.018e-01  1.519e-02   6.704 2.05e-11 ***\nmeta.strategy11:si:pi     1.053e-01  3.003e-02   3.507 0.000453 ***\nmeta.strategy12:si:pi     1.452e-01  2.480e-02   5.854 4.83e-09 ***\nmeta.strategy0:cs:ss      4.150e-02  5.702e-03   7.277 3.47e-13 ***\nmeta.strategy1:cs:ss      2.126e-02  4.197e-03   5.066 4.08e-07 ***\nmeta.strategy3:cs:ss      1.957e-02  8.250e-03   2.372 0.017716 *  \nmeta.strategy4:cs:ss      2.251e-02  8.522e-03   2.642 0.008248 ** \nmeta.strategy6:cs:ss     -2.080e-02  8.252e-03  -2.520 0.011725 *  \nmeta.strategy8:cs:ss      2.892e-02  6.146e-03   4.705 2.55e-06 ***\nmeta.strategy9:cs:ss     -1.160e-02  4.077e-03  -2.846 0.004427 ** \nmeta.strategy0:cs:ps      1.049e-02  1.643e-03   6.387 1.71e-10 ***\nmeta.strategy1:cs:ps      9.818e-03  8.975e-04  10.939  < 2e-16 ***\nmeta.strategy4:cs:ps      5.491e-03  1.558e-03   3.524 0.000426 ***\nmeta.strategy6:cs:ps      5.453e-03  1.685e-03   3.236 0.001212 ** \nmeta.strategy7:cs:ps      1.663e-02  5.796e-03   2.870 0.004111 ** \nmeta.strategy8:cs:ps      8.532e-03  1.196e-03   7.132 1.00e-12 ***\nmeta.strategy9:cs:ps      1.075e-02  9.312e-04  11.539  < 2e-16 ***\nmeta.strategy0:ss:ps      2.157e-02  3.442e-03   6.267 3.73e-10 ***\nmeta.strategy1:ss:ps      5.873e-03  2.428e-03   2.419 0.015550 *  \nmeta.strategy6:ss:ps     -2.724e-02  3.682e-03  -7.399 1.39e-13 ***\nmeta.strategy8:ss:ps      1.317e-02  2.168e-03   6.071 1.28e-09 ***\nmeta.strategy9:ss:ps     -2.022e-02  2.143e-03  -9.437  < 2e-16 ***\nmeta.strategy10:ss:ps    -2.329e-02  8.699e-03  -2.678 0.007418 ** \nmeta.strategy12:ss:ps    -4.750e-02  1.354e-02  -3.508 0.000453 ***\nmeta.strategy0:ca:sa:pa   7.122e-04  2.809e-04   2.536 0.011221 *  \nmeta.strategy1:ca:sa:pa   6.479e-04  1.704e-04   3.802 0.000143 ***\nmeta.strategy6:ca:sa:pa  -5.498e-04  2.101e-04  -2.617 0.008876 ** \nmeta.strategy8:ca:sa:pa   1.050e-03  1.848e-04   5.680 1.36e-08 ***\nmeta.strategy9:ca:sa:pa   9.064e-04  1.695e-04   5.347 8.99e-08 ***\nmeta.strategy12:ca:sa:pa -2.990e-03  1.176e-03  -2.541 0.011049 *  \nmeta.strategy0:ci:si:pi  -5.373e-04  1.504e-04  -3.573 0.000353 ***\nmeta.strategy1:ci:si:pi  -8.256e-04  1.165e-04  -7.087 1.39e-12 ***\nmeta.strategy2:ci:si:pi  -4.921e-04  1.883e-04  -2.613 0.008983 ** \nmeta.strategy4:ci:si:pi   6.554e-04  1.597e-04   4.104 4.07e-05 ***\nmeta.strategy6:ci:si:pi  -7.904e-04  1.646e-04  -4.801 1.58e-06 ***\nmeta.strategy7:ci:si:pi  -9.727e-04  3.325e-04  -2.926 0.003438 ** \nmeta.strategy8:ci:si:pi  -6.379e-04  1.254e-04  -5.085 3.68e-07 ***\nmeta.strategy10:ci:si:pi -1.726e-03  3.271e-04  -5.277 1.32e-07 ***\nmeta.strategy12:ci:si:pi -2.887e-03  8.524e-04  -3.387 0.000707 ***\nmeta.strategy0:cs:ss:ps  -7.405e-05  3.021e-05  -2.451 0.014231 *  \nmeta.strategy1:cs:ss:ps  -1.547e-04  1.813e-05  -8.533  < 2e-16 ***\nmeta.strategy3:cs:ss:ps  -1.752e-04  4.287e-05  -4.086 4.39e-05 ***\nmeta.strategy6:cs:ss:ps   2.174e-04  2.949e-05   7.373 1.70e-13 ***\nmeta.strategy8:cs:ss:ps  -8.523e-05  2.254e-05  -3.782 0.000156 ***\nmeta.strategy9:cs:ss:ps   1.547e-04  1.768e-05   8.753  < 2e-16 ***\nmeta.strategy10:cs:ss:ps  2.300e-04  7.745e-05   2.969 0.002985 ** \nmeta.strategy12:cs:ss:ps  6.302e-04  2.238e-04   2.817 0.004856 ** \n";


	strat_lm = new LM[num_strategies+1];
	for (int s=num_strategies; s>=0; s--)
	    {
		strat_lm[s] = new LM(modstring, String.format("meta.strategy%d", s), 
				     "ca sa pa ci si pi cs ss ps");
	    }

    }

    float[] estimatePayoffs ()
    {
	
    // public int oppCooperationMetric[] = new int[3];
    // public int oppSandwichMetric[] = new int[3];
    // public int oppPredictabilityMetric[] = new int[3];
	
	// order of variables:
	// "ca sa pa ci si pi cs ss ps"
// > d$ca = pmax(d$c.1, d$c.2)
// > d$sa = pmax(d$s.1, d$s.2)
// > d$pa = pmax(d$p.1, d$p.2)
// > d$ci = pmin(d$c.1, d$c.2)
// > d$si = pmin(d$s.1, d$s.2)
// > d$pi = pmin(d$p.1, d$p.2)
// > d$cs = d$c.1 + d$c.2
// > d$ss = d$s.1 + d$s.2
// > d$ps = d$p.1 + d$p.2

	float[] variables = {
	    max(oppCooperationMetric[1],oppCooperationMetric[2]),
	    max(oppSandwichMetric[1],oppSandwichMetric[2]),
	    max(oppPredictabilityMetric[1],oppPredictabilityMetric[2]),
	    min(oppCooperationMetric[1],oppCooperationMetric[2]),
	    min(oppSandwichMetric[1],oppSandwichMetric[2]),
	    min(oppPredictabilityMetric[1],oppPredictabilityMetric[2]),
	    (oppCooperationMetric[1]+oppCooperationMetric[2]),
	    (oppSandwichMetric[1]+oppSandwichMetric[2]),
	    (oppPredictabilityMetric[1]+oppPredictabilityMetric[2])};

	
	float[] payoff = new float[num_strategies+2];
	for (int s=num_strategies; s>=0; s--)
	    {
		payoff[s] = strat_lm[s].predict(variables);
	    }

	return payoff;
    }

}
