/* Authors : David Reitter
 */

package edu.cmu.psy.reitter.meta;

import game.Game;
import game.Strategy;
import game.Utility;
import java.util.Random;
import lemonade.LemonadeUtility;
import static java.lang.Math.*;
import java.io.DataInputStream;
import java.io.DataOutputStream;
import java.io.FileInputStream;
import java.io.FileOutputStream;
import java.io.IOException;
import java.io.FileNotFoundException;
import java.io.*;

import game.Strategy.*;

public class Taylor // implements Strategy
{

    final int NUM_STRATEGIES = 20;

    Utility utility;
    Random mRandom;

    public final int FIRST_PLAYER = 1;
    public final int SECOND_PLAYER = 2;
    public final int ME = 0;
    final int MAX_HISTORY_STORE = 105;
    public final int FIRST_LEVEL_HISTORY = 2;
    public final int NUM_PLAYERS = 3;

    int count = 0;
    int action[][] = new int [NUM_PLAYERS][MAX_HISTORY_STORE];
    int my_utility[][] = new int [NUM_PLAYERS][MAX_HISTORY_STORE];

    public int oppCooperationMetric[] = new int[3];
    public int oppSandwichMetric[] = new int[3];
    public int oppPredictabilityMetric[] = new int[3];

    public Taylor (Game g, long seed, String options) 
    {
	utility = g.getUtility();
	mRandom = new Random(seed);
    }


 
	public int playerAction(int num){
		int prev_pos = action[num][count-1];
		int d1[] = new int [FIRST_LEVEL_HISTORY];
		if (count == 2)
			return prev_pos;
		else{
			for (int i = 0;i < FIRST_LEVEL_HISTORY;i++)
				d1[i] = action[num][count - 2 - i] - action[num][count - 1 - i];	
			if (d1[0] == 0){
				return prev_pos;
			}
			else if (d1[0] == d1[1]){                                            
				return prev_pos + d1[0];
			}
			else{
				return prev_pos;
			}
		}
	}
    int norm (int pos)
    {
	
	pos = pos%12;
	if (pos<0)
	    pos += 12;

	return pos;
    }
  
    int playerDidCooperate (int player, int[] actions)
    {

	int op1, op2;

	if (player==1)
	    {op1 = 0; op2 = 2;}
	else if (player==2)
	    {op1 = 0; op2 = 1;}
	else
	    {op1 = 1; op2 = 2;}

	if (actions[player] == norm(actions[op2]+6))
	    return 1;
	if (actions[player] == norm(actions[op1]+6))
	    return 1;

	return 0;
    }

    int playerDidSandwich (int player, int[] actions)
    {

	int op1, op2;

	if (player==1)
	    {op1 = 0; op2 = 2;}
	else if (player==2)
	    {op1 = 0; op2 = 1;}
	else
	    {op1 = 1; op2 = 2;}

	if (actions[player] == norm(actions[op1]-1))
	    return 1;
	if (actions[player] == norm(actions[op1]+1))
	    return 1;
	if (actions[player] == norm(actions[op2]-1))
	    return 1;
	if (actions[player] == norm(actions[op2]+1))
	    return 1;

	return 0;
    }

    int playerDidActPredictably (int player, int[] actions)
    {
	// must be called before adding actions to history

	if (playerAction(player) == actions[player])
	    return 1;
	return 0;
	
    }

  // @Override
  // public int getAction() {
  //     return 0;
  // }
   
  
  
  /* Called at the end of each round returning the actions taken by the players,
     Stores the actions in three seperate integer arrays, and gets the utilities
     to be stored in seperate arrays for utilities */
  // @Override
    public void observeOutcome(int[] actions) {

	if (count>1)
	    {
		for (int i = 1; i<3; i++)
		    {
			oppCooperationMetric[i] += playerDidCooperate(i, actions);
			oppSandwichMetric[i] += playerDidSandwich(i, actions);
			oppPredictabilityMetric[i] += playerDidActPredictably(i, actions);
		    }
	    }

	count ++;
	action[ME][count-1]=actions[ME];
	action[FIRST_PLAYER][count-1]=actions[FIRST_PLAYER];
	action[SECOND_PLAYER][count-1]=actions[SECOND_PLAYER];
	my_utility[ME][count-1]=utility.getUtility(actions)[ME];
	my_utility[FIRST_PLAYER][count-1]=utility.getUtility(actions)[FIRST_PLAYER];
	my_utility[SECOND_PLAYER][count-1]=utility.getUtility(actions)[SECOND_PLAYER];
    }



}
