package analysis;
import java.util.Random;


public class GameAnalysis {
  

  public static double getFraction(int numActions, int numRepeats, GameSet gs, GameGeneratorInterface generator){
    Random r = new Random(0);
    int unsolvableCount=0;
    for(int i=0;i<numRepeats;i++){
      ThreePlayerGame g  = generator.generateGame(r,numActions);
      if (gs.contains(g)){
        unsolvableCount++;
      }
    }
    return ((double)unsolvableCount)/numRepeats;
    
  }

  /**
   * Run rotational 
   * @param args
   */
  public static void main(String[] args){
    //runColorActions(12,new TruncatedUniformDistribution());
    //runTests(new LemonadeGameGenerator());
    //runTests(new AsymmetricGameGenerator(396,true,new TrinomialUtilityDistribution()));
    runTests(new LocationGameGenerator());
    
  }
  

  public static void runRotationalSymmetry(){
    System.out.println("Fixed pairs, rotational");
    runTests(new RotationalGameGenerator(396,true,true));
    System.out.println("Random pairs, rotational");
    runTests(new RotationalGameGenerator(396,false,true));
  }

  /**
   * In this case, the actions are all unrelated, like colors.
   */
  public static void runColorActions(final int maxValue, final UtilityDistribution type){
    
    System.out.println("Fixed pairs, symmetric actions");
    runTests(new GameGenerator(maxValue,true,true,type));
    System.out.println("Random pairs, symmetric actions");
    runTests(new GameGenerator(maxValue,false,true,type));
    System.out.println("Fixed pairs, asymmetric actions");
    runTests(new GameGenerator(maxValue,true,false,type));
    System.out.println("Random pairs, asymmetric actions");
    runTests(new GameGenerator(maxValue,false,false,type));
    
  }
  
  
  public static void runTests(GameGeneratorInterface generator){
    System.out.println("Number of Actions\tFraction Pure Equilibria\tFraction Unsolvable\tFraction Colludable\tFraction Unsolvable and Ex Ante Colludable\tEx Ante Core Colludable\tUnsolvable and Ex Ante Core Colludable\tIf Pure, Unsolvable");
    for(int numActions=2;numActions<=12;numActions++){
      
      final double fractionPureEquilibria=getFraction(numActions,10000,new HasPureEquilibria(),generator);
      final double unsolvableCount=getFraction(numActions,10000, new Unsolvable(),generator);
      final double exAnteColludable = getFraction(numActions,10000,new ExAnteColludable(),generator);
      final double exAnteCoreColludable = getFraction(numActions,10000,new ExAnteCoreColludable(),generator);

      final double unsolvableAndColludable = getFraction(numActions,10000,new UnionGameSet(new ExAnteColludable(),new Unsolvable()),generator);
      final double unsolvableAndCoreColludable = getFraction(numActions,10000,new UnionGameSet(new ExAnteCoreColludable(),new Unsolvable()),generator);

      final double ifPureUnsolvable = unsolvableCount/fractionPureEquilibria;
      
      System.out.println(numActions+"\t"+fractionPureEquilibria+"\t"+unsolvableCount+"\t"+exAnteColludable+"\t"+unsolvableAndColludable+"\t"+exAnteCoreColludable+"\t"+unsolvableAndCoreColludable+"\t"+ifPureUnsolvable);
    }

  }
}
