package analysis;
import java.util.Random;


public class RotationalGameGenerator implements GameGeneratorInterface{
  int MAX_VALUE;
  boolean deterministicPairs;
  boolean symmetricDistribution;
  public RotationalGameGenerator(int MAX_VALUE,boolean deterministicPairs, boolean symmetricDistribution){
    this.MAX_VALUE=MAX_VALUE;
    this.deterministicPairs = deterministicPairs;
    this.symmetricDistribution = symmetricDistribution;
  }

  public RotationalGameGenerator(int MAX_VALUE,boolean deterministicPairs){
    this.MAX_VALUE=MAX_VALUE;
    this.deterministicPairs = deterministicPairs;
    this.symmetricDistribution = true;
  }


  public void setPermutation(ThreePlayerGame g, int[] a, int[] u,int i, int j, int k){
    int[] utility = g.getUtility(a[i],a[j],a[k]);
    
    utility[0]=u[i];
    utility[1]=u[j];
    utility[2]=u[k];
  }
  
  public void setJointActionSymmetrically(ThreePlayerGame g, int[] a, int[] u){
    setPermutation(g,a,u,0,1,2);
    setPermutation(g,a,u,0,2,1);
    setPermutation(g,a,u,1,0,2);
    setPermutation(g,a,u,1,2,0);
    setPermutation(g,a,u,2,0,1);
    setPermutation(g,a,u,2,1,0);
  }
  
  public void setJointActionRotationally(ThreePlayerGame g, int[] a, int[] u){
    int numActions = g.getNumActions()[0];
    for(int i=0;i<numActions;i++){
      for(int j=0;j<a.length;j++){
        a[j]=(a[j]+1)%numActions;
      }
      setJointActionSymmetrically(g,a,u);
    }
  }
  
  
  
  
  public void shuffle(int[] array, Random r){
    for(int i=0;i<array.length;i++){
      int target = r.nextInt(array.length-i)+i;
      if (target>i){
        final int temp=array[target];
        array[target]=array[i];
        array[i]=temp;
      }
    }
  }
  
  
  
  public void generatePairedActions(ThreePlayerGame g, Random r, int NUM_ACTIONS){
    int[] actions = new int[3];
    int[] values = new int[3];
    int i=0;
      for(int j=1;j<NUM_ACTIONS;j++){
        actions[0]=i;
        actions[1]=j;
        actions[2]=j;
        values[0]=r.nextInt(MAX_VALUE+1);
        values[1]=r.nextInt(MAX_VALUE+1-values[0]);
        values[2]=MAX_VALUE-values[1]-values[0];
        values[0] = values[r.nextInt(3)];
        int remaining = MAX_VALUE - values[0];
        if ((remaining & 1)==1){
          remaining--;
          values[0]++;
        }
        values[1]=remaining/2;
        values[2]=remaining/2;
        setJointActionRotationally(g,actions,values);
        
    }

  }
  

  
  public void generatePairedDeterministically(ThreePlayerGame g, int NUM_ACTIONS){
    int[] actions = new int[3];
    int[] values = new int[3];
    
    int i=0;
      for(int j=0;j<NUM_ACTIONS;j++){
        if (j==i){
          continue;
        }
        actions[0]=i;
        actions[1]=j;
        actions[2]=j;
        values[0]=MAX_VALUE/2;
        values[1]=MAX_VALUE/4;
        values[2]=MAX_VALUE/4;
        setJointActionRotationally(g,actions,values);
      }

  }
  
  
  
  
  public void generateUniqueActions(ThreePlayerGame g, Random r, int numActions){
    int[] actions = new int[3];
    int[] values = new int[3];
    int i=0;
      for(int j=i+1;j<numActions-1;j++){
        for(int k=j+1;k<numActions;k++){
          actions[0]=i;
          actions[1]=j;
          actions[2]=k;
          values[0]=r.nextInt(MAX_VALUE+1);
          values[1]=r.nextInt(MAX_VALUE+1-values[0]);
          values[2]=MAX_VALUE-values[1]-values[0];
          if (symmetricDistribution){
            shuffle(values,r);
          }
          setJointActionRotationally(g,actions,values);
        }
      }
  }
  
  public void generateSymmetricActions(ThreePlayerGame g, int numActions){
    final int AVERAGE_VALUE = MAX_VALUE/3;
    for(int i=0;i<numActions;i++){
      g.data[i][i][i][0]=AVERAGE_VALUE;
      g.data[i][i][i][1]=AVERAGE_VALUE;
      g.data[i][i][i][2]=AVERAGE_VALUE;
    }
  }

  public ThreePlayerGame generateGame(Random r, int numActions){
    ThreePlayerGame g = new ThreePlayerGame(numActions);
    
    generateUniqueActions(g,r,numActions);
    
    generateSymmetricActions(g,numActions);
    if (deterministicPairs){
      generatePairedDeterministically(g,numActions);
    } else {
      generatePairedActions(g,r,numActions);
    }
    return g;
  }
}
