package analysis;
import java.util.Arrays;


public class ThreePlayerGame {
  public ThreePlayerGame(int[][][][] data) {
    super();
    this.data = data;
  }


  public double getUtility(int action, double[] distribution){
    double value = 0;
    for(int actionB=0;actionB<data.length;actionB++){
      for(int actionC=0;actionC<data.length;actionC++){
        value += (double)(data[action][actionB][actionC][0]) * distribution[actionB]* distribution[actionC];
      }
    }
    return value;
  }
  
  
  public int[][][][] data;
  
  public String toString(){
    int[] numActions = getNumActions();
    StringBuilder builder = new StringBuilder(numActions[0]*numActions[1]*numActions[2]*20);
    for(int i=0;i<numActions[0];i++){
      for(int j=0;j<numActions[1];j++){
        for(int k=0;k<numActions[2];k++){
          int[] array = data[i][j][k];
          builder.append(i+" "+j+" "+k+" "+array[0]+" "+array[1]+" "+array[2]+"\n");
        }
      }
    }
    return builder.toString();
  }
  
  public int[] getUtility(int[] actions){
    return data[actions[0]][actions[1]][actions[2]];
  }
  

  public int[] getUtility(int i, int j, int k){
    return data[i][j][k];
  }
  
  public boolean isEquilibrium(int[] actions){
    int[] numActions = getNumActions();
    int[] changedActions = new int[actions.length];
    System.arraycopy(actions, 0, changedActions, 0, actions.length);
    int[] tenativeBest = getUtility(changedActions);
    for(int activePlayer=0;activePlayer<numActions.length;activePlayer++){
      int tenativeBestForPlayer = tenativeBest[activePlayer];
      for(int action=0;action<numActions[activePlayer];action++){
        changedActions[activePlayer]=action;
        int otherUtility = getUtility(changedActions)[activePlayer];
        if (otherUtility>tenativeBestForPlayer){
          return false;
        }
      }
      changedActions[activePlayer]=
        actions[activePlayer];
    }
    return true;
  }
  public int[] getNumActions(){
    int[] result = new int[3];
    result[0]=data.length;
    result[1]=data[0].length;
    result[2]=data[0][0].length;
    return result;
  }
  public ThreePlayerGame(int numActions){
    data = new int[numActions][numActions][numActions][3];
  }
  
  
  public void setUtility(int x, int y, int z, int []values){
    System.arraycopy(values,0,data[x][y][z],0,3);
  }
  
  

  public int[][] getPureEquilibria(){
    int[] actions = this.getNumActions();

    int countEquilibria = 0;
    int[][] equilibria = new int[actions[0]*actions[1]*actions[2]][];
  
  for(int i=0;i<actions[0];i++){
    for(int j=0;j<actions[1];j++){
      for(int k=0;k<actions[2];k++){
        int[] candidate = new int[3];
        candidate[0]=i;
        candidate[1]=j;
        candidate[2]=k;
        if (this.isEquilibrium(candidate)){
          equilibria[countEquilibria++]=candidate;
        }
      }
    }
  }
    int[][] result = new int[countEquilibria][];
    System.arraycopy(equilibria,0,result,0,countEquilibria);
    return result;
  }

}
