package lemonade;

import java.io.PrintStream;
import java.math.BigInteger;

import util.Array;

import game.Utility;

/**
 * An optimized version of the utility function. Note that this
 * requires 12^3 queries to startup, making it optimal only if more
 * than 1728 (or even 2000) queries are needed.
 * 
 * The utilities are also scaled down to be the 
 * @author maz
 *
 */
public class ScaledLemonadeUtility implements Utility{
  /**
   * The number of actions.
   */
  int numActions;
  
  /**
   * The number of players.
   */
  static int numPlayers=3;
  
  /**
   * The utility.
   */
  int [][][][]utility;
  
  
  /**
   * The scale of the utilities.
   */
  int scale;
  
  
  /**
   * Gets the scale factor between LemonadeUtility and
   * ScaledLemonadeUtility.
   * @return the scale
   */
  public int getScale(){
    return scale;
  }
  /**
   * Get the size of the island.
   * @return the size of the island.
   */
  public int getIslandSize(){
    return numActions;
  }
  
  /**
   * Construct a lemonade utility function.
   * @param numActions
   */
  public ScaledLemonadeUtility(int numActions){
    this.numActions = numActions;
    utility = new int[numActions][numActions][numActions][numPlayers];
    init();
    
  }
  
  /**
   * Initialize utility.
   */
  public void init(){
    LemonadeUtility lemonadeUtility = new LemonadeUtility(numActions);
    int[] actions = new int[numPlayers];
    for(int i=0;i<numActions;i++){
      for(int j=0;j<numActions;j++){
        for(int k=0;k<numActions;k++){
          actions[0]=i;
          actions[1]=j;
          actions[2]=k;
          lemonadeUtility.getUtility(actions,utility[i][j][k]);
        }
      }
    }
    
    scale = getGCD();
    for(int i=0;i<numActions;i++){
      for(int j=0;j<numActions;j++){
        for(int k=0;k<numActions;k++){
          for(int m=0;m<numPlayers;m++){
            utility[i][j][k][m]/=scale;
          }
        }
      }
    }
  }

  /**
   * Print out the utilities.
   * Print out the utilities in the form:
   * 
   * <ACTION1> <ACTION2> <ACTION3> <UTILITY1> <UTILITY2> <UTILITY3>
   * @param out
   */
  public void dumpUtilities(PrintStream out){
    for(int i=0;i<numActions;i++){
      for(int j=0;j<numActions;j++){
        for(int k=0;k<numActions;k++){
          out.println(i+" "+j+" "+k+" "+Array.arrayToNakedString(utility[i][j][k]));
        } 
      } 
    }
    
  }
  private int getGCD(){
    BigInteger currentGCD = new BigInteger(""+utility[0][0][0][0]);
    for(int i=0;i<numActions;i++){
      for(int j=0;j<numActions;j++){
        for(int k=0;k<numActions;k++){
          for(int m=0;m<numPlayers;m++){
            BigInteger currentUtility = new BigInteger(""+utility[i][j][k][m]);
            currentGCD = currentGCD.gcd(currentUtility);
          }
        } 
      } 
    }
    return currentGCD.intValue();
  }
 
  /**
   * Get the utility.
   * Returns a copy of the data in the directory.
   */
  public void getUtility(int[] actions,int[] result){
    assert(actions.length==numPlayers);
    assert(result.length==numPlayers);
    int[] resultInternal = utility[actions[0]][actions[1]][actions[2]];
    System.arraycopy(resultInternal, 0, result, 0, numPlayers);
  }

  /**
   * Get the utility.
   * Constructs a new array and returns that.
   */
  public int[] getUtility(int[] actions){
    int[] result = new int[numPlayers];
    getUtility(actions,result);
    return result;
  }

  public boolean isSymmetric(){
    return true;
  }
  
  public Utility swapPlayers(int i,int j){
    return this;
  }
}
