package net;

import java.awt.AWTException;
import java.awt.BorderLayout;
import java.awt.Robot;
import java.awt.event.InputEvent;
import java.io.IOException;
import java.net.ServerSocket;
import java.net.Socket;
import java.net.SocketException;

import javax.swing.JFrame;

import game.Game;
import human.HumanDisplay;
import human.HumanModelData;
import lemonade.ScaledLemonadeUtility;

public class TestHumanRemoteClient {
  
  public static void checkMessage(LineSocket socket,String expected) throws SocketException, IOException{
    String message = socket.receiveMessage();
    if (!message.equals(expected)){
      throw new RuntimeException("Expected "+expected+" Received "+message);
    }
  }
  
  
  public static void checkMessageState(HumanRemoteClient client, String expected){
    HumanModelData data = new HumanModelData(3);
    
    client.model.getModel(data);
    if (!expected.equals(data.message)){
      throw new RuntimeException("Expected:"+expected+" actual:"+data.message);
    }
  }
  
  /**
   * This moves the mouse to the proper coordinates and clicks the
   * button. Works only on my machine.
   * @param r
   */
  public static void clickQuit(Robot r){
    r.mouseMove(689,418);
    r.mousePress(InputEvent.BUTTON1_MASK);
    r.mouseRelease(InputEvent.BUTTON1_MASK);
  }

  /**
   * This moves the mouse to the proper coordinates and clicks the
   * button. Works only on my machine.
   * @param r
   */
  public static void clickPlayAgain(Robot r){
    r.mouseMove(597,403);
    r.mousePress(InputEvent.BUTTON1_MASK);
    r.mouseRelease(InputEvent.BUTTON1_MASK);
    
  }
  
  public static void checkHandshake(LineSocket link, ServerSocket server, HumanRemoteClient client) throws IOException, InterruptedException{
    link.openConnection(server.accept());
    
    // This crazy wait is because in a previous version, the code would
    // change to "Choose an action..." before the game had begun.
    Thread.sleep(6000);
    checkMessageState(client,"Waiting for other players...");
    
    link.sendMessage("PING");
    checkMessage(link,"ACK");
    
    link.sendMessage("BEGIN:LEMONADE 3 12 0");
    Thread.sleep(1000);
    checkMessageState(client,"Begin playing!");
  }
  
  public static void main(String[] args) throws IOException, InterruptedException, AWTException{
    ServerSocket server = new ServerSocket(5000);

    Game g = new Game(new int[] { 12, 12, 12 }, new ScaledLemonadeUtility(12));
    HumanDisplay hdisplay = new HumanDisplay(g);
    HumanStrategy constantStrategy = new HumanStrategy(g, hdisplay);
    HumanRemoteClient client = new HumanRemoteClient(constantStrategy,g);
    ChatPanel panel = new ChatPanel(client);
    
    hdisplay.addMouseListener(constantStrategy);
    JFrame frame = new JFrame("Lemonade Stand Game");
    frame.setDefaultCloseOperation(JFrame.EXIT_ON_CLOSE);
    frame.setLayout(new BorderLayout());
    frame.add(hdisplay,BorderLayout.CENTER);
    frame.add(panel,BorderLayout.SOUTH);
    frame.pack();
    frame.validate();
    frame.setLocationRelativeTo(null);
    frame.setVisible(true);

    /*Game g = new Game(new int[] { 12, 12, 12 }, new ScaledLemonadeUtility(12));
    HumanStrategy constantStrategy = new HumanStrategy(g, 0, "");
    HumanRemoteClient client = new HumanRemoteClient(constantStrategy,g);*/
    
    
    Thread t = new Thread(client);
    t.start();
    Thread.sleep(1000);
    checkMessageState(client,"Waiting for other players...");
    
    
    Socket s = server.accept();
    Thread.sleep(1000);
    
    checkMessageState(client,"Waiting for other players...");
    
    LineSocket link = new LineSocket();
    link.socketName = "SERVER";
    link.openConnection(s);
    Thread.sleep(1000);
    link.sendMessage("PING");
    checkMessage(link,"ACK");
    
    checkMessageState(client,"Waiting for other players...");
    
    
    link.sendMessage("BEGIN:LEMONADE 3 12 0");
    Thread.sleep(1000);
    
    checkMessageState(client,"Begin playing!");
    panel.entry.setText("Hello");
    panel.entry.postActionEvent();
    
    checkMessage(link,"CHAT:Hello");
    
    Thread.sleep(1000);
    constantStrategy.clickOnLocation(6);
    Thread.sleep(1000);
    checkMessageState(client,"Action locked in...");
    
    
    
    
    checkMessage(link,"6");
    link.sendMessage("CHAT:Hello");
    link.sendMessage("OUTCOME:6 0 9");
    Thread.sleep(1000);
    checkMessageState(client,"Points won:9");
    
    System.out.println(panel.display.getText());
    
    Thread.sleep(1000);
    constantStrategy.clickOnLocation(5);
    checkMessage(link,"5");
    link.sendMessage("OUTCOME:5 6 7");
    Thread.sleep(1000);
    checkMessageState(client,"Points won:11");
    
    Thread.sleep(5000);
    checkMessageState(client,"Choose an action...");
    
    constantStrategy.clickOnLocation(5);
    checkMessage(link,"5");
    link.sendMessage("OUTCOME:5 8 11");
    Thread.sleep(7000);
    checkMessageState(client,"Choose an action...");

    
    
    
    constantStrategy.clickOnLocation(7);
    checkMessage(link,"7");
    link.sendMessage("GAMEOVER:7 5 3");
    Thread.sleep(1000);
    checkMessageState(client,"Game Over!");

    link.closeConnection();
    
    Robot r = new Robot();
    
    //Rectangle rect = new Rectangle(1280,800);
    //BufferedImage image = r.createScreenCapture(null);
    
    clickPlayAgain(r);
    
    link.openConnection(server.accept());
    
    // This crazy wait is because in a previous version, the code would
    // change to "Choose an action..." before the game had begun.
    Thread.sleep(6000);
    checkMessageState(client,"Waiting for other players...");
    
    link.sendMessage("PING");
    checkMessage(link,"ACK");
    
    link.sendMessage("BEGIN:LEMONADE 3 12 0");
    Thread.sleep(1000);
    checkMessageState(client,"Begin playing!");
    
    Thread.sleep(5000);
    checkMessageState(client,"Choose an action...");
    constantStrategy.clickOnLocation(3);
    checkMessage(link,"3");

    link.sendMessage("ABORT");
    Thread.sleep(1000);
    checkMessageState(client,"Game Over!");
    
    
    clickPlayAgain(r);
    link.openConnection(server.accept());
    link.sendMessage("ABORT");
    Thread.sleep(1000);
    checkMessageState(client,"Waiting for other players...");
    

    link.openConnection(server.accept());
    link.sendMessage("PING");
    checkMessage(link,"ACK");

    link.sendMessage("BEGIN:LEMONADE 3 12 0");
    Thread.sleep(1000);
    constantStrategy.clickOnLocation(3);
    link.sendMessage("3 11 7");
    Thread.sleep(1000);
    checkMessageState(client,"Game Over!");

    // This tests that the client handles an action out of range
    clickPlayAgain(r);
    link.openConnection(server.accept());
    
    link.sendMessage("PING");
    checkMessage(link,"ACK");
    
    link.sendMessage("BEGIN:LEMONADE 3 12 0");
    Thread.sleep(1000);
    constantStrategy.clickOnLocation(3);
    link.sendMessage("OUTCOME:3 12 7");
    Thread.sleep(1000);
    checkMessageState(client,"Game Over!");

    clickPlayAgain(r);
    link.openConnection(server.accept());
    link.sendMessage("PING");
    checkMessage(link,"ACK");
    
    link.sendMessage("BEGIN:LEMONADE 3 12 0");
    Thread.sleep(1000);
    link.sendMessage("ABORT");
    Thread.sleep(1000);
    checkMessageState(client,"Game Over!");
    
    
    clickQuit(r);
    System.err.println("Test reached completion...");
    t.interrupt();
    frame.dispose();
    
    System.exit(0);
    
    
  }
}
