package results;

import java.io.File;
import java.io.IOException;
import java.util.Hashtable;
import java.util.Map;
import java.util.Random;
import java.util.Vector;

import util.Array;

public class AlternativeCompetitionsFast {
  
  
  EmpiricalGame foundation;
  String[] allStrategies;
  String[] allPlayers;
  
  double[][] weights;
  int draws;
  Random r;
  
  public AlternativeCompetitionsFast(EmpiricalGame foundation, String[] allStrategies, int numPlayers){
    this.foundation = foundation;
    this.allStrategies = allStrategies;
    initializePlayers(numPlayers);
    weights = new double[numPlayers][allStrategies.length];
    initializeWeights();
    draws = 1;
    r = new Random(0);
  }
  
  
  public double getFitness(int action, double[] distribution){
    setWeights(distribution);
    double rawUtility = this.getExpectedUtility(0, action);
    return rawUtility + 99.0;
  }

  public void setWeights(double[] distribution){
    for(double[] playerWeights:weights){
      System.arraycopy(distribution, 0, playerWeights, 0, distribution.length);
      for(int j=distribution.length;j<playerWeights.length;j++){
        playerWeights[j]=0;
      }
    }
  }
  
  
  public void setWeights(double[] newDistribution, double[] oldDistribution){
    System.arraycopy(newDistribution, 0, weights[0], 0, newDistribution.length);
    for(int i=1;i<weights.length;i++){
      double[] playerWeights=weights[i];
      System.arraycopy(oldDistribution, 0, playerWeights, 0, oldDistribution.length);
      for(int j=oldDistribution.length;j<playerWeights.length;j++){
        playerWeights[j]=0;
      }
    }
  }

  public double getFitnessNew(int action, double[] newDistribution, double[] oldDistribution){
    setWeights(newDistribution,oldDistribution);
    
    double rawUtility = this.getExpectedUtility(0, action);
    return rawUtility + 99.0;
  }

  public double getFitnessOld(int action, double[] newDistribution, double[] oldDistribution){
    setWeights(newDistribution,oldDistribution);
    
    double rawUtility = this.getExpectedUtility(1, action);
    return rawUtility + 99.0;
  }

  
  
  
  private static void uniformDist(double[] distribution){
    for(int action=0;action<distribution.length;action++){
      distribution[action]=1.0/distribution.length;
    }
  }
  
  public double[][] getEquilibrium(int oldPlayers, int iterations){
    double[] newDistribution = new double[allStrategies.length];
    double[] oldDistribution = new double[oldPlayers];
    uniformDist(newDistribution);
    uniformDist(oldDistribution);
    update(newDistribution,oldDistribution,iterations);
    return new double[][]{newDistribution,oldDistribution};
  }
  
  public double[][] getEquilibriumSynced(int oldPlayers, int iterations){
    double[] newDistribution = new double[allStrategies.length];
    double[] oldDistribution = new double[oldPlayers];
    uniformDist(newDistribution);
    uniformDist(oldDistribution);
    update(newDistribution,oldDistribution,iterations);
    return new double[][]{newDistribution,oldDistribution};
  }
  
  
  public double [] getEquilibrium(int iterations){
    double[] distribution = new double[allStrategies.length];
    uniformDist(distribution);
    update(distribution,iterations);
    return distribution;
  }
  
  public void update(double[] distribution, int iterations){
    
    double[] fitness = new double[distribution.length];
    for(int i=0;i<iterations;i++){
    for(int action=0;action<fitness.length;action++){
      fitness[action]=getFitness(action,distribution);
    }
    updateWithFitness(distribution,fitness);
    }
  }

  private static void updateWithFitness(double[] distribution, double[] fitness){
    double total = 0;
    for(int action=0;action<fitness.length;action++){
      distribution[action]=distribution[action]*fitness[action];
      total+=distribution[action];
    }
    for(int action=0;action<fitness.length;action++){
      distribution[action]/=total;
    }
    
  }
  public void update(double[] newDistribution, double[] oldDistribution, int iterations){
    
    double[] newFitness = new double[newDistribution.length];
    for(int i=0;i<iterations;i++){
    for(int action=0;action<newFitness.length;action++){
      newFitness[action]=getFitnessNew(action,newDistribution,oldDistribution);
    }
    
    double[] oldFitness = new double[oldDistribution.length];
    for(int action=0;action<oldFitness.length;action++){
      oldFitness[action]=getFitnessOld(action,newDistribution,oldDistribution);
    }

    updateWithFitness(oldDistribution,oldFitness);
    updateWithFitness(newDistribution,newFitness);
    }
  }

  
  public int drawStrategy(int playerIndex){
    double dart = r.nextDouble();
    double[] playerWeights = weights[playerIndex];
    for(int i=0;i<playerWeights.length;i++){
      dart-=playerWeights[i];
      if (dart<=0){
        return i;
      }
    }
    return playerWeights.length-1;
    
  }

  public double getExpectedUtility(int playerIndex, int strategyIndex){
    double total = 0;
    for(int i=0;i<draws;i++){
      total += getUtilityDraw(playerIndex,strategyIndex);
    }
    return total/draws;
  }
  
  
  public double getUtilityDraw(int playerIndex, int strategyIndex){
    int[] strategies = new int[allPlayers.length];
    for(int i=0;i<strategies.length;i++){
      if (i==playerIndex){
        strategies[i]=strategyIndex;
      } else {
        strategies[i]=drawStrategy(i);
      }
    }
    return getUtility(playerIndex,strategies);
  }
  
  

  /**
   * Simulate the competition, but map players to strategies.
   * Every three players (not including repeats) are played.
   * No variance reduction tricks are used, nor is variance calculated.
   */
  public long[] runAsymmetricNonSelfPlayTournament(int[] strategies){
    long[] results = new long[strategies.length];
    
    for(int i=0;i<strategies.length;i++){
      for(int j=0;j<strategies.length;j++){
        for(int k=0;k<strategies.length;k++){
          if (i!=j && i!=k && j!=k){
            
            long[] result = foundation.utility[strategies[i]][strategies[j]][strategies[k]];
            results[i]+=result[0];
            results[j]+=result[1];
            results[k]+=result[2];
            
          }
        }
      }
    }
    return results;
  }
  
  public void compareDeviation(){
    for(int deviation=0;deviation<allStrategies.length;deviation++){
      for(int begin=0;begin<allStrategies.length;begin++){
        double utility = getUtility(deviation,begin);
        System.out.println("Initial:"+allStrategies[begin]+" Deviation:"+allStrategies[deviation]+" "+utility);
      }
    }
  }
  
  public double getUtility(int deviation, int begin){
    int[] strategies = new int[allPlayers.length];
    strategies[0]=deviation;
    for(int i=1;i<strategies.length;i++){
      strategies[i]=begin;
    }
    double utility = getUtility(0,strategies);
    return utility;
  }
  
  public double getUtility(int playerIndex, int[] strategies){
    String[] stringStrategies = new String[strategies.length];
    for(int i=0;i<strategies.length;i++){
      stringStrategies[i]=allStrategies[strategies[i]];
    }
    long[] results= runAsymmetricNonSelfPlayTournament(strategies);
    long maxScore = Long.MIN_VALUE;
    long maxScoreRepeat = 0;
    
    for(long currentScore:results){
      if (currentScore>maxScore){
        maxScore = currentScore;
        maxScoreRepeat = 1;
      } else if (currentScore==maxScore){
        maxScoreRepeat++;
      }
    }
    long playerScore = results[playerIndex];
    if (playerScore<maxScore){
      return 0;
    }
    return 1.0/maxScoreRepeat;
  }
  
  private void initializePlayers(int number){
    allPlayers = new String[number];
    for(int i=0;i<allPlayers.length;i++){
      allPlayers[i]="player"+i;
    }
  }
  
  private void initializeWeights(){
    for(double[] playerWeights:weights){
      double average = 1.0/((double)playerWeights.length);
      for(int i=0;i<playerWeights.length;i++){
        playerWeights[i]=average;
      }
    }
  }

  public static void showUsage(){
    System.err.println("Usage:AlternativeCompetitions [-data <DATAFILE>| -matchtype <MATCHTYPE>]* -players [<CLASS> <OPTION>]*");
    System.exit(0);
  }

  public double[] getInitialDist(String map){
    String[] tokens = map.split("\\[|\\]|\\,");
    Hashtable<String,Double> mapToDistribution = new Hashtable<String,Double>();
    for(String token:tokens){
      if (token.length()!=0){
        String[] tokens2 = token.split("=");
        if (tokens2.length==2){
          mapToDistribution.put(tokens2[0], Double.parseDouble(tokens2[1]));
        }
      }
    }
    double[] result = new double[allStrategies.length];
    for(int i=0;i<allStrategies.length;i++){
      if (mapToDistribution.containsKey(allStrategies[i])){
        result[i]=mapToDistribution.get(allStrategies[i]);
      }
    }
    return result;
  }
  
  public static void main(String[] args) throws IOException{
    
    if (args.length<4){
      showUsage();
    }
    
    int repeats = 1000;
    File dataFile = null;
    int iterations = 100;
    int oldPlayers = 0;
    int totalPlayers = 0;
    int playerArgIndex=args.length;
    String initialDist = null;
    String analysis = "default";
    for(int i=0;i<args.length;i+=2){
      String arg = args[i];
      String fieldStr = args[i+1];
      if (arg.equals("-method")){
        analysis = fieldStr;
      } else if (arg.equals("-data")){
        dataFile = new File(fieldStr);
      } else if (arg.equals("-iterations")){
        iterations = Integer.parseInt(fieldStr);
      } else if (arg.equals("-totalplayers")){
        totalPlayers = Integer.parseInt(fieldStr);
      } else if (arg.equals("-eval")){
        initialDist = fieldStr;
      } else if (arg.equals("-repeats")){
        long field = Long.parseLong(fieldStr);
        if ((long)(Integer.MAX_VALUE)<field){
          System.err.println("Number of repeats needs to be less than 2^31-1");
          showUsage();
        }
        repeats = (int)field;
      } else if (arg.equals("-oldplayers")){
        oldPlayers = Integer.parseInt(fieldStr);
      } else if (arg.equals("-players")){
        playerArgIndex=i+1;
        break;
      } else {
        System.err.println("Unknown option:"+arg);
        showUsage();
        
      }
    }
    if (dataFile==null){
      showUsage();
    }

    String[] strategies = new String[(args.length-playerArgIndex)/2];
    String[] options = new String[strategies.length];
    int playerIndex = 0;
    for(int i=playerArgIndex;i<args.length;i+=2){
      strategies[playerIndex]=args[i];
      options[playerIndex]=args[i+1];
      playerIndex++;
    }
    
    EmpiricalGame tournament = EmpiricalGame.getAsymmetricGame(TripleResults.getResultsWithVariance(dataFile,repeats),strategies);
    
    
    
    if (totalPlayers==0){
      totalPlayers = strategies.length;
    }
    AlternativeCompetitionsFast competitions = new AlternativeCompetitionsFast(tournament,strategies, totalPlayers);
    if (analysis.equals("compare")){
      competitions.compareDeviation();
      return;
    } else if (analysis.equals("syncold")){
      
    } else if (analysis.equals("testbeat")){
      int[] strategyIndices = new int[oldPlayers+1];
      for(int i=0;i<oldPlayers;i++){
        strategyIndices[i]=i;
      }
      for(int i=oldPlayers;i<strategies.length;i++){
        strategyIndices[oldPlayers]=i;
        double util = competitions.getUtility(oldPlayers, strategyIndices);
        System.out.println(strategies[strategyIndices[oldPlayers]]+":"+util);
      }
      return;
    }
    if (initialDist!=null){
      double[] distribution = competitions.getInitialDist(initialDist);
      competitions.setWeights(distribution);
      competitions.draws = 100;
      double[] utilities = new double[competitions.allStrategies.length];
      for(int i=0;i<utilities.length;i++){
        utilities[i] = competitions.getExpectedUtility(0, i);
      }
      String result = ConvertToJordan.prettyPrintArrays(strategies, utilities);
      System.out.println(result);
    } else if (oldPlayers==0){
      double[] distribution = competitions.getEquilibrium(iterations);
      System.out.println(Array.arrayToNakedString(distribution));
    } else {
      double[][] distribution = competitions.getEquilibrium(oldPlayers,iterations);
      String[] oldStrategies = new String[oldPlayers];
      System.arraycopy(strategies, 0, oldStrategies, 0, oldStrategies.length);
      System.out.println(ConvertToJordan.prettyPrintEquilibrium(strategies, distribution[0], 0.0));
      System.out.println(ConvertToJordan.prettyPrintEquilibrium(oldStrategies, distribution[1], 0.0));
      competitions.setWeights(distribution[1]);
      double[] utilities = new double[strategies.length];
      for(int i=0;i<utilities.length;i++){
        utilities[i] = competitions.getExpectedUtility(0, i);
      }
      System.out.println(ConvertToJordan.prettyPrintArrays(strategies, utilities));
      
      
    }
  }
  
}
