package results;

import java.io.File;
import java.io.IOException;
import java.util.Arrays;
import java.util.Hashtable;
import java.util.Vector;

import analysis.FitnessFunction;
import analysis.ThreePlayerGame;

public class ConvertToJordan {
  
  public static void selfPlayGuess(Vector<TripleResults> results){
    TripleResults first = results.firstElement();
    long totalUtility = first.scores[0]+first.scores[1]+first.scores[2];
    long average = totalUtility/3;
    long averagePerRepeat = average/first.repeats;
    long squaredAverage = (averagePerRepeat*averagePerRepeat) * first.repeats;
    String[] names = TripleResults.getNames(results);
    long[] squaredScores=new long[3];
    long[] scores = new long[3];
    
    for(int i=0;i<3;i++){
      scores[i]=average;
      squaredScores[i]=squaredAverage;
    }
    for(String name:names){
      String[] nameTriple = new String[3];

      for(int i=0;i<3;i++){
        nameTriple[i]=name;
      }
      TripleResults result = new TripleResults(nameTriple,scores,squaredScores,first.repeats);
      results.add(result);
    }
    
  }
  
  public static Vector<TripleResults> getResults(File file, File mappingFile) throws IOException{
    Vector<TripleResults> results = TripleResults.getResultsWithVariance(file,30);
    Hashtable<String,String> mapping = new Hashtable<String,String>();
    TripleResults.loadHashtable(mappingFile, mapping);
    TripleResults.translate(results, mapping);
    
    selfPlayGuess(results);
    return results;
  }
  
  public static String prettyPrintArrays(String[] names, double[] values){
    StringBuilder result = new StringBuilder();
    result.append('[');
    boolean firstShown = false;
    for(int i=0;i<names.length;i++){
        if (firstShown){
          result.append(", ");
        }
        result.append(names[i]);
        result.append('=');
        result.append(values[i]);
        firstShown=true;
    }
    result.append(']');
    return result.toString();
    
  }
  public static String prettyPrintEquilibrium(String[] names, double[] probs, double tolerance){
    StringBuilder result = new StringBuilder();
    result.append('[');
    boolean firstShown = false;
    for(int i=0;i<names.length;i++){
      if (probs[i]>=tolerance){
        if (firstShown){
          result.append(", ");
        }
        result.append(names[i]);
        result.append('=');
        result.append(probs[i]);
        firstShown=true;
      }
    }
    result.append(']');
    return result.toString();
  }
  
  public static void evaluateResults(Vector<TripleResults> results){
    EmpiricalGame game = new EmpiricalGame(results);
    //game.write(System.out);
    ThreePlayerGame tpgame = game.getGame();
    int[][] eqa = tpgame.getPureEquilibria();
    System.out.println("Pure equilibria:"+eqa.length);
    for(int[] eqm:eqa){
      String[] namedEqa = new String[eqm.length];
      for(int i=0;i<eqm.length;i++){
        namedEqa[i]=game.actionNames[eqm[i]];
      }
      System.out.println(Arrays.toString(namedEqa));
    }
    FitnessFunction function = new FitnessFunction(tpgame);
    double[] equilibrium = function.getSolution();
    System.out.println(Arrays.toString(game.actionNames));
    System.out.println("Equilibrium:"+prettyPrintEquilibrium(game.actionNames,equilibrium,0.0));
    
    System.out.println("Regret:"+function.getSolutionRegret(equilibrium));
    System.out.println("Util vector:"+prettyPrintArrays(game.actionNames,function.getUtilVector(equilibrium)));
  }

  
  public static void removeAndCompare(Vector<TripleResults> results, String[] names, int player0, int player1){
    EmpiricalGame original = new EmpiricalGame(results);
    Vector<TripleResults> results2 = new Vector<TripleResults>(results);
    KnockoutTable.removePlayer(results2,names[player0]);
    KnockoutTable.removePlayer(results2,names[player1]);
    EmpiricalGame restricted = new EmpiricalGame(results2);
    ThreePlayerGame tpg = restricted.getGame();
    FitnessFunction functionRestricted = new FitnessFunction(tpg);
    double[] distribution = functionRestricted.getSolution();
    
    double[] expandedDist  = original.expandDistribution(restricted,distribution);
    ThreePlayerGame tpgOriginal = original.getGame();
    double util0 = tpgOriginal.getUtility(player0,expandedDist);
    double util1 = tpgOriginal.getUtility(player1, expandedDist);
    
    //System.out.println(names[player0]+":"+util0+","+names[player1]+":"+util1);
    if (util0>util1){
      System.out.println(names[player0]+":"+util0+","+names[player1]+":"+util1);
    } else if (util0<util1){
      System.out.println(names[player1]+":"+util1+","+names[player0]+":"+util0);
    } else {
      System.out.println(names[player0]+":"+util0+","+names[player1]+":"+util1+",TIE");      
    }
    
    
  }
  public static void removeEachAndPlay(Vector<TripleResults> results){
    String[] names = TripleResults.getNames(results);
    
    for(String name:names){
      Vector<TripleResults> copy = new Vector<TripleResults>(results);
      KnockoutTable.removePlayer(copy, name);
      System.out.println("Removing "+name+"...");
      evaluateResults(copy);
    }
  }
  public static void removeAndCompare(Vector<TripleResults> results){
    String[] names = TripleResults.getNames(results);
    for(int i=0;i<names.length;i++){
      for(int j=i+1;j<names.length;j++){
        removeAndCompare(results,names,i,j);
      }
      
    }
  }
  
  public static void main(String[] args) throws IOException{
    File file = new File(args[0]);
    File mappingFile = new File(args[1]);
    Vector<TripleResults> results = getResults(file,mappingFile);
    //removeAndCompare(results);
    removeEachAndPlay(results);
    System.out.println("With everyone...");
    evaluateResults(results);
  }
}
