package results;

import java.io.PrintStream;
import java.util.Hashtable;
import java.util.Vector;

import analysis.ThreePlayerGame;


public class EmpiricalGame {
  long[][][][] utility;
  String[] actionNames;
  Hashtable<String,Integer> mapping;
  
  
  
  
  public EmpiricalGame(String[] actions){
    this.actionNames = actions;
    mapping = new Hashtable<String,Integer>();
    for(int i=0;i<actions.length;i++){
      mapping.put(actions[i], i);
    }
    utility=new long[actionNames.length][actionNames.length][actionNames.length][3];
  }

  public EmpiricalGame(Vector<TripleResults> results){
    this(TripleResults.getNames(results));
    for(TripleResults triple:results){
      setTriple(triple);
    }
    
  }
 
  public static EmpiricalGame getAsymmetricGame(Vector<TripleResults> results, String[] actions){
    EmpiricalGame result = new EmpiricalGame(actions);
    for(TripleResults triple:results){
      result.writeAsymmetricTriple(triple);
    }
    return result;
  }
  
  public void writeAsymmetricTriple(TripleResults results){
    int[] actions = getActions(results);
    long[] utilities = getUtilities(results);
    writePermutedTriple(actions,utilities,0,1,2);
  }
  
  private void writePlayer(PrintStream out, String name){
    out.println("  <player id=\""+name+"\">");
    out.println("    <actions>");
    for(String action:actionNames){
      out.println("      <action id=\""+action+"\"/>");
    }
    out.println("    </actions>");
    out.println("  </player>");
  }
  
  public void writePayoff(PrintStream out, String player, String action, long payoff){
    out.println("<outcome player=\""+player+"\" action=\""+action+"\""+" value="+payoff+"/>");
  }
  
  
  public void write(PrintStream out){
    out.println("<?xml version=\"1.0\" encoding=\"utf-8\" ?>");
    out.println("<nfg name=\"TEST\" description=\"TEST\">");
    out.println("<players>");
    writePlayer(out,"alice");
    writePlayer(out,"bob");
    writePlayer(out,"cathy");
    out.println("</players>");
    out.println("<payoffs>");
    for(int i=0;i<actionNames.length;i++){
      for(int j=0;j<actionNames.length;j++){
        for(int k=0;k<actionNames.length;k++){
          long[] profile=utility[i][j][k];
          out.println("<payoff>");
          writePayoff(out,"alice",actionNames[i],profile[0]);
          writePayoff(out,"bob",actionNames[j],profile[1]);
          writePayoff(out,"cathy",actionNames[k],profile[2]);
          out.println("</payoff>");
          
        }
      }
    }
    out.println("</payoffs>");

    out.println("</nfg>");
  }

  private static long[] getUtilities(TripleResults triple){
    int numPlayers = triple.scores.length;
    long[] utilities = new long[numPlayers];
    System.arraycopy(triple.scores, 0, utilities, 0, numPlayers);
    return utilities;
  }
  
  private int[] getActions(TripleResults triple){
    int numPlayers = triple.playerNames.length;
    int[] actions = new int[numPlayers];
    for(int i=0;i<triple.playerNames.length;i++){
      actions[i]=mapping.get(triple.playerNames[i]);
    }
    return actions;
  }
  public void setTriple(TripleResults triple){
    int[] actions = getActions(triple);
    long[] utilities = getUtilities(triple);
    for(int i=0;i<utilities.length;i++){
      utilities[i]*=2;
    }
    if (actions[0]==actions[1]){
      long average = (utilities[0]+utilities[1])/2;
      utilities[0]=utilities[1]=average;
    }
    writePermutedTriple(actions,utilities,0,1,2);
    writePermutedTriple(actions,utilities,0,2,1);
    writePermutedTriple(actions,utilities,1,0,2);
    writePermutedTriple(actions,utilities,1,2,0);
    writePermutedTriple(actions,utilities,2,0,1);
    writePermutedTriple(actions,utilities,2,1,0);
  }
  
  private void writePermutedTriple(int[] actions,long[] utilities, int a, int b, int c){
    long[] target = utility[actions[a]][actions[b]][actions[c]];
    target[0]=utilities[a];
    target[1]=utilities[b];
    target[2]=utilities[c];
  }
  
  public ThreePlayerGame getGame(){
    int[][][][] result = new int[utility.length][utility.length][utility.length][3];
    for(int i=0;i<result.length;i++){
      for(int j=0;j<result[i].length;j++){
        for(int k=0;k<result[i][j].length;k++){
          for(int m=0;m<result[i][j][k].length;m++){
            result[i][j][k][m]=(int)(utility[i][j][k][m]);
          }
        }
      }
    }
    return new ThreePlayerGame(result);
  }

  public double[] expandDistribution(EmpiricalGame restricted,double[] distribution) {
    double[] result = new double[actionNames.length];
    for(int i=0;i<distribution.length;i++){
      int index = mapping.get(restricted.actionNames[i]);
      result[index]=distribution[i];
    }
    return result;
  }
}
