package results;
import java.io.BufferedReader;
import java.io.File;
import java.io.FileReader;
import java.io.IOException;
import java.util.HashSet;
import java.util.Hashtable;
import java.util.Vector;

import util.Array;


public class TripleResults {
  public TripleResults(String[] playerNames, long[] scores,
      long[] squaredScores, int repeats) {
    super();
    this.playerNames = playerNames;
    this.scores = scores;
    this.squaredScores = squaredScores;
    this.repeats = repeats;
  }

  public String[] playerNames;
  public long[] scores;
  public long[] squaredScores;
  int repeats;
  
  
  
  public TripleResults(String playerLine, String scoreLine){
    playerNames = playerLine.split(" ");
    String[] scoreStrings = scoreLine.split(" ");
    if (scoreStrings.length!=playerNames.length){
      throw new RuntimeException("Number of players does not equal the number of scores:"+playerLine+"!="+scoreLine);
    }
    scores = new long[scoreStrings.length];
    for(int i=0;i<scores.length;i++){
      scores[i]=Long.parseLong(scoreStrings[i]);
    }
    
  }
  
  public void add(TripleResults other){
    assert(samePlayersSameOrder(other));
    for(int i=0;i<scores.length;i++){
      scores[i]+=other.scores[i];
    }
    for(int i=0;i<squaredScores.length;i++){
      squaredScores[i]+=other.squaredScores[i];
    }
  }
  

  public boolean samePlayersSameOrder(String[] otherplayerNames){
    if (playerNames.length!=otherplayerNames.length){
      return false;
    }
    for(int i=0;i<playerNames.length;i++){
      if (!playerNames[i].equals(otherplayerNames[i])){
        return false;
      }
    }
    return true;
  }
  
  public boolean samePlayersSameOrder(TripleResults other){
    return samePlayersSameOrder(other.playerNames);
  }
  
  public String toString(){
    return Array.arrayToNakedString(playerNames)+"\n"+
    Array.arrayToNakedString(scores)+"\n"+
    Array.arrayToNakedString(squaredScores);
  }
  
  private static double getStandardErrorOfTheMean(long score, long squaredScore, int repeats){
    double mean = ((double)score)/repeats;
    double meanOfSquares = ((double)squaredScore)/repeats;
    double biasedVariance = meanOfSquares - (mean * mean);
    double unbiasedVariance = (biasedVariance * repeats)/(repeats-1);
    double oneExampleStandardDeviation = Math.sqrt(unbiasedVariance);
    double standardErrorOfTheMean = oneExampleStandardDeviation/Math.sqrt(repeats);
    return standardErrorOfTheMean;
  }
  
  /**
   * Returns the standard deviations for all the players.
   * @return
   */
  public double[] getStandardErrorOfTheMean(){
    double[] result = new double[scores.length];
    for(int i=0;i<result.length;i++){
      result[i]=getStandardErrorOfTheMean(scores[i],squaredScores[i],repeats);
    }
    return result;
  }
  
  
  public double[] getMeans(){
    double[] means = new double[scores.length];
    for(int i=0;i<means.length;i++){
      means[i]=(double)scores[i]/repeats;
    }
    return means;
  }
  public TripleResults(String playerLine, String scoreLine, String squaredScoreLine, int repeats){
    this(playerLine,scoreLine);
    String[] scoreStrings = squaredScoreLine.split(" ");
    if (scoreStrings.length!=playerNames.length){
      throw new RuntimeException("Number of players does not equal the number of scores:"+playerLine+"!="+scoreLine);
    }
    squaredScores = new long[scoreStrings.length];
    for(int i=0;i<squaredScores.length;i++){
      squaredScores[i]=Long.parseLong(scoreStrings[i]);
    }
    this.repeats = repeats;
    
  }
  
  
  
  /**
   * Returns the index of the player with name in the game,
   * or -1 if the player is not present.
   * @param name the name of the player we are searching for.
   * @return -1 if not present, otherwise 0, 1, or 2
   */
  public int getIndex(String name){
    for(int i=0;i<playerNames.length;i++){
      if (name.equals(playerNames[i])){
        return i;
      }
    }
    return -1;
  }
  
  
  public static String[] getNames(Vector<TripleResults> results){
    HashSet<String> allNames = new HashSet<String>();
    for(TripleResults result:results){
      for(String name:result.playerNames){
        allNames.add(name);
      }
    }
    String[] result = new String[allNames.size()];
    
    allNames.toArray(result);
    return result;
  }
  public static void loadHashtable(File file, Hashtable<String,String> mapping) throws IOException{
    BufferedReader reader = new BufferedReader(new FileReader(file));
    while(true){
      String line = reader.readLine();
      if (line==null){
        return;
      }
      String[] tokens = line.split(" ");
      if (tokens.length!=2){
        continue;
      }
      mapping.put(tokens[0], tokens[1]);
    }
  }
  
  public void translate(Hashtable<String,String> nameMapping){
    for(int i=0;i<playerNames.length;i++){
      if (nameMapping.containsKey(playerNames[i])){
        playerNames[i]=nameMapping.get(playerNames[i]);
      }
    }
  }
  
  public static void translate(Vector<TripleResults> results, Hashtable<String,String> nameMapping){
    for(TripleResults result:results){
      result.translate(nameMapping);
    }
  }
  
  
  public static Vector<TripleResults> getResults(File f) throws IOException{
    BufferedReader reader = new BufferedReader(new FileReader(f));
    Vector<TripleResults> results = new Vector<TripleResults>();
    while(true){
      String playerLine=reader.readLine();
      if (playerLine==null){
        break;
      }
      String scoreLine=reader.readLine();
      if (scoreLine==null){
        break;
      }
      results.add(new TripleResults(playerLine,scoreLine));
    }
    return results;
    
  }

  private static void merge(Vector<TripleResults> current, TripleResults next){
    for(TripleResults oldTriple:current){
      if (oldTriple.samePlayersSameOrder(next)){
        oldTriple.add(next);
        return;
      }
    }
    current.add(next);
  }
  
  public static Vector<TripleResults> getResultsWithVariance(File f, int repeats) throws IOException{
    BufferedReader reader = new BufferedReader(new FileReader(f));
    Vector<TripleResults> results = new Vector<TripleResults>();
    while(true){
      String playerLine=reader.readLine();
      if (playerLine==null){
        break;
      }
      String scoreLine=reader.readLine();
      if (scoreLine==null){
        break;
      }
      String squaredScoreLine=reader.readLine();
      if (squaredScoreLine==null){
        break;
      }
      TripleResults newTriple = new TripleResults(playerLine,scoreLine,squaredScoreLine,repeats);
      merge(results,newTriple);
    }
    return results;
  }

}
