package stats.collusion;

import game.Game;


/**
 * An observer of utility, like jointactionutility, that does
 * not keep track of joint actions, simply utilities.
 * @author maz
 *
 */
public class PseudoPairwiseCollusionObserver extends AbstractPseudoCollusionObserver{

  int[][] collusionsStarted;
  int[][] collusionsBroken;
  int[][] totalCollusionLength;
  int[][][] collusionBreaker;
  int[] pseudoCollusionsStarted;
  int[] pseudoCollusionsBroken;
  int[] totalPseudoCollusionLength;
  
  
  
  public void observeStartRealCollusion(){
    int player0 = this.collusionPlayers[0];
    int player1 = this.collusionPlayers[1];
    this.collusionsStarted[player0][player1]++;
    this.totalCollusionLength[player0][player1]++;
  }
  public void observeStartCollusion(){
    if (collusionPlayers.length==2){
      observeStartRealCollusion();
    } else {
      observeStartPseudoCollusion();
    }
  }
  
  
  
  private void observeStartPseudoCollusion() {
    int player0 = this.collusionPlayers[0];
    this.pseudoCollusionsStarted[player0]++;
    this.totalPseudoCollusionLength[player0]++;
  }
  public PseudoPairwiseCollusionObserver(Game g){
    this(g.getNumPlayers());
  }
  
  
  /**
   * Construct a utility observer.
   * @param utility a utility function to be used.
   * @param numPlayers the number of players in the game.
   */
  public PseudoPairwiseCollusionObserver(int numPlayers){
    this.collusionsStarted = new int[numPlayers][numPlayers];
    this.collusionsBroken = new int[numPlayers][numPlayers];
    this.totalCollusionLength = new int[numPlayers][numPlayers];
    this.totalPseudoCollusionLength = new int[numPlayers];
    this.collusionBreaker = new int[numPlayers][numPlayers][numPlayers+1];
    pseudoCollusionsStarted = new int[numPlayers];
    pseudoCollusionsBroken = new int[numPlayers];
    
    AbstractCollusionObserver.MIN_LENGTH = 1;
  }
  

  public void observeTerminateCollusion(boolean countBroken){
    if (collusionPlayers.length==2){
      observeTerminateRealCollusion(countBroken);
    } else {
      observeTerminatePseudoCollusion(countBroken);
    }
  }
  
  
  public void observeTerminatePseudoCollusion(boolean countBroken){
    int player = collusionPlayers[0];
    if (this.collusionDidNotCount()){
      pseudoCollusionsStarted[player]--;
      totalPseudoCollusionLength[player]-=getCollusionLength();
    } else {
      pseudoCollusionsBroken[player]++;
    }
  }
  
  public void observeTerminateRealCollusion(boolean countBroken){
    int player0 = this.collusionPlayers[0];
    int player1 = this.collusionPlayers[1];

    int collusionLength = this.getCollusionLength();
    if (this.collusionDidNotCount()){
      // A one-round collusion doesn't count
      collusionsStarted[player0][player1]--;
      totalCollusionLength[player0][player1]-=collusionLength;
    } else if (countBroken){
      collusionsBroken[player0][player1]++;
      boolean player0Breaks = (this.collusionActions[player0]!=this.actions[player0]);
      boolean player1Breaks = (this.collusionActions[player1]!=this.actions[player1]);
      if (player0Breaks && (!player1Breaks)){
        collusionBreaker[player0][player1][player0]++;
      }
      if (player1Breaks && (!player0Breaks)){
        collusionBreaker[player0][player1][player1]++;
      }
      if (player0Breaks && (player1Breaks)){
        collusionBreaker[player0][player1][3]++;
      }
    }
  }
  
  
  public void observeContinueRealCollusion(){
    int player0 = this.collusionPlayers[0];
    int player1 = this.collusionPlayers[1];

    this.totalCollusionLength[player0][player1]++;
  }
  
  public void observeContinuePseudoCollusion(){
    int player0 = this.collusionPlayers[0];
    this.totalPseudoCollusionLength[player0]++;
  }

  public void observeContinueCollusion(){
    if (collusionPlayers.length==1){
      observeContinuePseudoCollusion();
    } else {
      observeContinueRealCollusion();
      
    }
  }
  
  public int[][] getCollusionsStarted(){
    return collusionsStarted;
  }
  public int[][] getCollusionsBroken(){
    return collusionsBroken;
  }
  
  public int[][] getTotalCollusionLength(){
    return totalCollusionLength;
  }

  public static String getString(int[] counts){
    StringBuilder builder = new StringBuilder("[");
    if (counts.length>=1){
      builder.append(counts[0]);
    }
    
    for(int j=1;j<counts.length;j++){
      builder.append(","+counts[j]);
    }
    builder.append("]");
    return builder.toString();
  }

  public static String getString(int[][] counts){
    StringBuilder builder = new StringBuilder("[");
    if (counts.length>=1){
      builder.append(getString(counts[0]));
    }
    for(int j=1;j<counts.length;j++){
      builder.append(","+getString(counts[j]));
    }
    builder.append("]");
    return builder.toString();
  }

  public static String getString(int[][][] counts){
    StringBuilder builder = new StringBuilder("[");
    if (counts.length>=1){
      builder.append(getString(counts[0]));
    }
    for(int j=1;j<counts.length;j++){
      builder.append(","+getString(counts[j]));
    }
    builder.append("]");
    return builder.toString();
  }
  
  /**
   * Display the utilities for the series and the squares of the utilities.
   */
  public void display(String[] playerNames) {
    System.out.print("{");
    System.out.print("\"all\":[" + playerNames[0]+","+playerNames[1]+","+playerNames[2]+"],");
    System.out.print("\"collusionsStarted\":"+getString(collusionsStarted)+",");
    System.out.print("\"collusionsBroken\":"+getString(collusionsBroken)+",");
    System.out.print("\"totalCollusionLength\":"+getString(totalCollusionLength)+",");
    System.out.print("\"collusionBreaker\":"+getString(collusionBreaker)+",");
    System.out.print("\"pseudoCollusionsStarted\":"+getString(pseudoCollusionsStarted)+",");
    System.out.print("\"pseudoCollusionsBroken\":"+getString(pseudoCollusionsBroken)+",");
    System.out.println("\"totalPseudoCollusionLength\":"+getString(totalPseudoCollusionLength)+"}");
  }
  @Override
  public void startGame(Game g) {
    // TODO Auto-generated method stub
    
  }
  


}
