package stats.collusion;

import stats.SeriesObserver;
import game.Game;


/**
 * An observer of utility, like jointactionutility, that does
 * not keep track of joint actions, simply utilities.
 * @author maz
 *
 */
public class SandwichCollusionObserver implements SeriesObserver{
  int[] lastActions;
  int[] thisActions;
  
  int[] constantSandwiched;
  int[] suddenlySandwiched;
  
  int numActions;
  boolean firstRound;
  
  public SandwichCollusionObserver(Game g){
    
    if (g.getNumPlayers() != 3){
      throw new RuntimeException("There are not three players");
    }
    
    numActions  = (g.getNumActions());
    lastActions = new int[3];
    thisActions = new int[3];
    constantSandwiched = new int[3];
    suddenlySandwiched = new int[3];
    
    firstRound = true;
  }
  
  
    
  
  
  
  
  public static final int MAX_DIFFERENCE=1;
  public boolean isNext(int a, int b){
    int difference = (b+12-a) % 12;
    return (difference>0) && (difference<=MAX_DIFFERENCE);
  }
  public boolean isSequential(int a, int b, int c){
    return isNext(a,b) && isNext(b,c);
  }
  
  
  public boolean isCurrentlySandwiched(int player){
    int playerAction = thisActions[player];
    int nextPlayerAction = thisActions[(player+1) % 3];
    int nextNextPlayerAction = thisActions[(player+2) % 3];
    return 
      isSequential(nextPlayerAction,playerAction,nextNextPlayerAction) ||
      isSequential(nextNextPlayerAction,playerAction,nextPlayerAction);
  }

  public void observeOutcome(int[] actions){
    System.arraycopy(thisActions, 0, lastActions, 0, thisActions.length);
    System.arraycopy(actions, 0, thisActions, 0, thisActions.length);
    
    for(int i=0;i<thisActions.length;i++){
      if (isCurrentlySandwiched(i)){
        if ((!firstRound) && (thisActions[i]==lastActions[i])){
          this.constantSandwiched[i]++;
        } else {
          this.suddenlySandwiched[i]++;
        }
      }
    }
    firstRound = false;
  }






  public static String getString(int[] counts){
    StringBuilder builder = new StringBuilder("[");
    if (counts.length>=1){
      builder.append(counts[0]);
    }
    
    for(int j=1;j<counts.length;j++){
      builder.append(","+counts[j]);
    }
    builder.append("]");
    return builder.toString();
  }


  @Override
  public void display(String[] playerNames) {
    System.out.print("{playerNames:["+playerNames[0]+","+playerNames[1]+","+playerNames[2]+"],");
    System.out.print("constantSandwiched:"+getString(constantSandwiched)+",");
    System.out.println("suddenlySandwiched:"+getString(suddenlySandwiched)+"}");
  }








  @Override
  public void finishGame() {
    // TODO Auto-generated method stub
    firstRound = true;
  }








  @Override
  public void finishSeries() {
    // TODO Auto-generated method stub
    
  }
  @Override
  public void startGame(Game g) {
    // TODO Auto-generated method stub
    
  }
  
}
