package strategy;

import game.Game;
import game.Strategy;

import java.util.Random;
import java.util.Vector;

import lemonade.GeneralizedLemonadeUtility;



/**
 * A strategy that chooses a random action on the first round, and then
 * plays that action thereafter.
 * @author maz
 *
 */
public class FancyConstantStrategy implements Strategy{
  /**
   * The action the player plays
   */
  int constantAction;
  
  int[] bestActions;
  
  
  private void initBestActions(GeneralizedLemonadeUtility utility){
    int[] density = utility.getIslandDensity();
    int maxDensity = density[0];
    int maxDensityCount = 1;
    for(int i=0;i<density.length;i++){
      if (maxDensity<density[i]){
        maxDensity=density[i];
        maxDensityCount=1;
      } else if (maxDensity==density[i]){
        maxDensityCount++;
      }
    }
    bestActions = new int[maxDensityCount];
    int bestActionIndex = 0;
    for(int i=0;i<density.length;i++){
      if (density[i]==maxDensity){
        bestActions[bestActionIndex]=i;
        bestActionIndex++;
      }
    }
  }
  /**
   * Construct a constant strategy. The game is
   * used to get the number of actions, and the
   * seed is used to seed a pseudorandom number 
   * generator, which then is queried for the action 
   * to take.
   * 
   * @param g
   * @param seed
   */
  public FancyConstantStrategy(Game g, long seed, String options) {
    Random r = new Random(seed);
    initBestActions((GeneralizedLemonadeUtility)g.getUtility());
    constantAction = bestActions[r.nextInt(bestActions.length)];
  }
  



  public int getAction() {
    return constantAction;
  }

  public void observeOutcome(int[] actions) {
  }
  
  
  
}
